use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $AmazonPlatformAlias = '';
    my $ShopAlias;
    my $Help;
    my $NoOutput;
    my $Reports;
    my $EndDate;
    my $StartDate;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
        'reports=s@'  => \$Reports,
        'startdate=s'  => \$StartDate,
        'enddate=s'  => \$EndDate,

    );

    usage() if $Help;
    usage() unless $StoreName;

    usage() if( $Reports && ($StartDate || $EndDate) );
    usage() if ( ($StartDate || $EndDate) and not($StartDate && $EndDate) );


    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            CheckGetOrders( {
                    'PrintOutput' => ($NoOutput) ? 0 : 1,
                    'AmazonPlatformAlias' => $AmazonPlatformAlias,
                    'ShopAlias'           => $ShopAlias,
                    'StartDate'           => $StartDate,
                    'EndDate'             => $EndDate,

                    # only query reports if shopalias and platform are given.
                    'Reports' => ( ( defined $ShopAlias and defined $AmazonPlatformAlias ) ? $Reports : undef ),
                    'StoreName' => $StoreName,
                }
            );
        },
    );
    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $logmsg  = sprintf( 'need %.3f seconds', $seconds );
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Amazon Order Documents: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub CheckGetOrders {    ## no critic (ProhibitFunctionSize)
    my ($hInput) = @_;

    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $hInput->{'StoreName'} . ": Cartridge Amazon is not installed.\n" if ( $hInput->{'PrintOutput'} );
        return;
    }

    my @AmazonConfigs     = ();
    my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');
    if ( defined $hInput->{'ShopAlias'} and $hInput->{'ShopAlias'} ne 'all' ) {
        my $Shop;
        eval { $Shop = LoadObjectByPath('/Shops')->child( $hInput->{'ShopAlias'} ); };
        if ( ExistsError() ) {
            printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", $hInput->{'StoreName'}, $hInput->{'ShopAlias'} );
            return;
        }

        my $aPortalConfigs = $Shop->get('PortalConfigs');
        @AmazonConfigs = grep { $_->instanceOf($AmazonConfigClass) } @{$aPortalConfigs};
    }
    else {
        @AmazonConfigs = @{ $AmazonConfigClass->objects };
    }

    # check if feature is enabled.
    my $Feature = LoadFeatureByAlias('Amazon');
    foreach my $Merchant (@AmazonConfigs) {
        my $PortalSite   = $Merchant->get('PortalSite');
        my $Shop         = $Merchant->getSite;
        my $FeatureValue = $Shop->featureMaxValue($Feature);

        next if ( not $FeatureValue );
        # skip if user requested a particular site.
        next if ( $hInput->{'AmazonPlatformAlias'} && $hInput->{'AmazonPlatformAlias'} ne $PortalSite->alias );
        # skip if shop is ClosedByProvider
        next if ($Shop->get('ClosedByProvider'));
        # skip if no merchant id is set or shop is still on SOAP

        # skip if no SPAPI_AccessToken is set
        next unless defined $Merchant->get('SPAPI_AccessToken');

        my $aWarnings = BlockWarnings(
            sub {

                print 'Get orders for shop: '. $Shop->alias.' | platform: '.$PortalSite->alias."\n" if $hInput->{'PrintOutput'};
                eval { $Merchant->getOrderDocuments( $hInput ); };

                if ( ExistsError() ) {
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $Merchant->pathString });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning ( @{$aWarnings} ) {
            LogError($Warning);
            # ignore failed logins in scheduler calls (still in the logs).
            next if ($Warning->code eq 'AmazonMerchantLoginFailure' && !$hInput->{'PrintOutput'});

            # ignore missing fees/currencies in scheduler mails (still in the logs).
            next if ($Warning->code eq 'AmazonOrder_NoFeeFound' && !$hInput->{'PrintOutput'});
            next if ($Warning->code eq 'AmazonOrder_NoCurrencyFound' && !$hInput->{'PrintOutput'});

            printf STDERR ( "[%s] WARN: %s: %s\n",
                $hInput->{'StoreName'},
                $Warning->vars->{Path}//'path unknown' ,
                substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias             (optional)   (all)
    -platform     amazon platform        (optional)   (Amazon_de_DE)
    -reports      query specific reports (optional)   ()
                  ! space-separated values
                  ! you can only query specific reports
                  ! if platform and shopalias are given.
                  ! -startdate and -enddate are not allowed together with this parameter
    -startdate    start date (ISO 8601) for fetching order documents (optional but mandatory if -enddate is given)
    -enddate      end date (ISO 8601) for fetching order documents (optional but mandatory if -startdate is given)
                  ! use UTC time for startdate and enddate as ISO 8601 value. Have a look to http://www.timestamp-converter.com/
                  for converting your TZ to UTC
flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE -reports 7923482 -reports 7934582
    perl $0 -storename Store -startdate 2022-07-27T07:00:00.000Z -enddate 2022-07-27T08:00:00.000Z
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE -startdate 2022-07-27T07:00:00.000Z -enddate 2022-07-27T08:00:00.000Z
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
