use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( Transaction LoadObject LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Amazon::API::Constants  qw ( %AMAZON_FEEDS );
use DE_EPAGES::Amazon::DAL::Amazon qw( DAL_GetAllPortalConfigIDsOfAmazonSubmitFeedQueue );


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $AmazonPlatformAlias = '';
    my $ShopAlias;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
    );

    usage() if $Help;
    usage() unless $StoreName;
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { processSubmitFeedQueue( ($NoOutput) ? 0 : 1, $AmazonPlatformAlias, $ShopAlias, $StoreName ) },
    );

    my $logmsg = sprintf 'need %.1f seconds', time() - $time1;
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Amazon SubmitFeedQueue: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub processSubmitFeedQueue {    ## no critic (ProhibitFunctionSize)
    my ( $printOutput, $AmazonPlatformAlias, $ShopAlias, $StoreName ) = @_;

    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $StoreName . ": Cartridge Amazon is not installed.\n" if ($printOutput);
        return;
    }

    my $aAmazonConfigIDs  = ();

    if ( defined $ShopAlias and $ShopAlias ne 'all' ) {
        my $Shop;
        eval { $Shop = LoadObjectByPath('/Shops')->child($ShopAlias); };
        if ( ExistsError() ) {
            printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias );
            return;
        }
        my $aPortalConfigs = $Shop->get('PortalConfigs');
        $aAmazonConfigIDs = [map { $_->id } grep { $_->instanceOf('AmazonConfig') } @{$aPortalConfigs}];
    }
    else {
      $aAmazonConfigIDs  = DAL_GetAllPortalConfigIDsOfAmazonSubmitFeedQueue();
    }

    # check if feature is enabled.
    my $Feature = LoadFeatureByAlias('Amazon');


    foreach my $MerchantID (@$aAmazonConfigIDs) {
        my $Merchant = LoadObject($MerchantID);
        my $PortalSite   = $Merchant->get('PortalSite');
        my $Shop         = $Merchant->getSite;
        my $FeatureValue = $Shop->featureMaxValue($Feature);
        next if ( not $FeatureValue );

        # skip if shop is ClosedByProvider
        next if ($Shop->get('ClosedByProvider'));

        # skip if user requested a particular site.
        next if ( $AmazonPlatformAlias && $AmazonPlatformAlias ne $PortalSite->alias );

        # skip if no SPAPI_AccessToken is set
        next unless defined $Merchant->get('SPAPI_AccessToken');

        my $aWarnings = BlockWarnings(
            sub {
                print 'Process SubmitFeedQueue for shop: '. $Shop->alias.' | platform: '.$PortalSite->alias if $printOutput;

                   eval { _ProcessSubmitFeedQueueForMerchant($Merchant,$printOutput); };

                if ( ExistsError() ) {
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $Merchant->pathString });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning ( @{$aWarnings} ) {
            # ignore failed logins in scheduler calls.
            next if (ElementOf($Warning->code, qw(AmazonMerchantLoginFailure AccessDenied InvalidMarketplace)) && !$printOutput);
            LogError($Warning);
            printf STDERR ( "[%s] WARN: %s: %s\n",
                $StoreName,
                $Warning->vars->{Path} // 'path unknown',
                substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }
}

sub _ProcessSubmitFeedQueueForMerchant(){
    my ( $Merchant, $printOutput ) = @_;
    my $Class  = LoadClassByAlias('AmazonSubmitFeedQueue');
    my $printMessageType = ' | MessageTypes:';

    my @queuedSubmitFeeds = qw ( POST_PRODUCT_DATA
    POST_INVENTORY_AVAILABILITY_DATA POST_PRODUCT_PRICING_DATA POST_ORDER_FULFILLMENT_DATA
    POST_ORDER_ACKNOWLEDGEMENT_DATA );


    foreach my $queuedSubmitFeed (@queuedSubmitFeeds) {
        my $aItems = $Class->getObjects( 'ByPortalConfigIDFeedType',[ $Merchant->id, $queuedSubmitFeed ] );
        my $XML = '';
        my $MessageID = 1;
        my $MessageType = '';
        my @Offers = ();
        my @Offers_Update = ();
        my @Offers_Delete = ();
        foreach my $submitFeedEntry (@$aItems) {
            my $OperationType = $submitFeedEntry->get('OperationType');
            my $OperationTypeXML = '';
            if(defined $submitFeedEntry->get('OperationType')) {
                $OperationTypeXML = '<OperationType>'.$submitFeedEntry->get('OperationType').'</OperationType>';
            }
            my $XMLMessageSnippet = $submitFeedEntry->get('XMLMessageSnippet');
            $MessageType = $submitFeedEntry->get('MessageType');
            push( @Offers, $submitFeedEntry->get('AmazonOffer') );

            if($OperationType eq "Update") {  push( @Offers_Update, $submitFeedEntry->get('AmazonOffer') ); }
            if($OperationType eq "Delete") {  push( @Offers_Delete, $submitFeedEntry->get('AmazonOffer') ); }

            $XML .= <<END_XML;
<Message>
  <MessageID>$MessageID</MessageID>
  $OperationTypeXML
  $XMLMessageSnippet
</Message>
END_XML

            $MessageID++;
        }
        if($MessageID > 1) {
            # Process the request
            my $FeedID = $Merchant->postDocument( $XML, $MessageType, $queuedSubmitFeed );

            Transaction( sub {
                print "$printMessageType $MessageType" if $printOutput;
                $printMessageType = '';

                # Price Update
                if (
                    $queuedSubmitFeed eq 'POST_PRODUCT_PRICING_DATA'
                   ) {
                    $Merchant->saveUploadMessages( $MessageType, $AMAZON_FEEDS{$queuedSubmitFeed},
                        $FeedID, \@Offers );
                }

                # Start offer & Remove offer
                if (
                    $queuedSubmitFeed eq 'POST_PRODUCT_DATA'
                   ) {
                        #Start Offer
                        $Merchant->saveUploadMessages( $MessageType, $AMAZON_FEEDS{$queuedSubmitFeed},
                            $FeedID, \@Offers_Update );

                        #Delete Offer
                        my @Products_Delete = ();
                        foreach my $Offer (@Offers_Delete) {
                            my $Product = $Offer->get('Product');
                            push( @Products_Delete, $Product ) if ( defined $Product );
                            $Offer->delete;
                        }

                        # remove portal assignment.
                        $Merchant->removeProducts( \@Products_Delete );
                   }

                # Remove entries from submitfeedqueue
                $Class->deleteObjects( 'ByPortalConfigIDFeedType',[ $Merchant->id, $queuedSubmitFeed ] );
            });
        }
    }

    print "\n" if $printOutput;

}


sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias               (optional) ()
    -platform     amazon platform          (optional) (Amazon_de_DE)

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
