use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $AmazonPlatformAlias = '';
    my $ShopAlias;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'platform=s'  => \$AmazonPlatformAlias,
        'shop=s'      => \$ShopAlias,
    );

    usage() if $Help;
    usage() unless $StoreName;
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { UpdateStock( ($NoOutput) ? 0 : 1, $AmazonPlatformAlias, $ShopAlias, $StoreName ) },
    );

    my $logmsg = sprintf 'need %.1f seconds', time() - $time1;
    print $logmsg. "\n" unless $NoOutput;
    GetLog()->debug("Amazon StockUpdate: $logmsg");

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub UpdateStock { ## no critic (ProhibitFunctionSize)
    my ( $printOutput, $AmazonPlatformAlias, $ShopAlias, $StoreName ) = @_;

    # make sure that the cartridge is installed
    unless ( ExistsCartridgeByPackage('DE_EPAGES::Amazon') ) {
        print 'Store: ' . $StoreName . ": Cartridge Amazon is not installed.\n" if ($printOutput);
        return;
    }

    my @AmazonConfigs;
    my $AmazonConfigClass = LoadClassByAlias('AmazonConfig');

    # check if feature is enabled.
    my $Feature = LoadFeatureByAlias('Amazon');

    if ( defined $ShopAlias and $ShopAlias ne 'all' ) {
        my $Shop;
        eval { $Shop = LoadObjectByPath('/Shops')->child($ShopAlias); };
        if ( ExistsError() ) {
            printf STDERR ( "[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias );
            return;
        }
        my $aPortalConfigs = $Shop->get('PortalConfigs');
        @AmazonConfigs = grep { $_->instanceOf($AmazonConfigClass) } @{$aPortalConfigs};
    }
    else {
        @AmazonConfigs = @{ $AmazonConfigClass->objects };
    }

    foreach my $Merchant (@AmazonConfigs) {
        my $PortalSite   = $Merchant->get('PortalSite');
        my $Shop         = $Merchant->getSite;
        my $FeatureValue = $Shop->featureMaxValue($Feature);
        next if ( not $FeatureValue );

        # skip if shop is ClosedByProvider
        next if ($Shop->get('ClosedByProvider'));

        # skip if user requested a particular site.
        next if ( $AmazonPlatformAlias && $AmazonPlatformAlias ne $PortalSite->alias );

        # skip if no SPAPI_AccessToken is set
        next unless defined $Merchant->get('SPAPI_AccessToken');

        my $aWarnings = BlockWarnings(
            sub {
                print 'Update stock for shop: '. $Shop->alias.' | platform: '.$PortalSite->alias."\n" if $printOutput;
                eval { $Merchant->uploadStock( $Merchant->folder('AmazonOffers')->children ); };

                if ( ExistsError() ) {
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $Merchant->pathString });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning (@{$aWarnings}) {
            # ignore failed logins in scheduler calls.
            next if ($Warning->code eq 'AmazonMerchantLoginFailure' && !$printOutput);
            LogError($Warning);
            printf STDERR ( "[%s] WARN: %s: %s \n",
                $StoreName,
                $Warning->vars->{Path}//'path unknown',
                substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Store)
    -shop         shop alias               (optional) ()
    -platform     amazon platform          (optional) (Amazon_de_DE)

flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop -platform Amazon_de_DE
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
