#========================================================================================
# package      DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for CreditCheckProvider.
#========================================================================================
package DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( Compare GetCurrentDateTime isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $CreditCheckProvider->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'CreditCheckProvider' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $CreditCheckProvider->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CreditCheckProvider = DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider->insert($hInfo);
# example      my $Class = LoadClassByAlias('CreditCheckProvider');
#               my $CreditCheckProvider = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertCreditCheckProvider' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $CreditCheckProvider | new object | object
# hook         OBJ_InsertCreditCheckProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CreditCheckProviderID - creditcheckprovider id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertCreditCheckProvider', {'Object' => $self, 'CreditCheckProviderID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Feature ConfigClass Name Image Logging ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CreditCheckProvider->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteCreditCheckProvider' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteCreditCheckProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CreditCheckProviderID - creditcheckprovider id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteCreditCheckProvider', {'Object' => $self, 'CreditCheckProviderID' => $self->id });
    LoadClassByAlias('CreditCheckAgency')->deleteObjects('ByCreditCheckProviderID', [$self->id]);
    LoadClassByAlias('CreditCheckProviderConfig')->deleteObjects('ByCreditCheckProviderID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CreditCheckProvider->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateCreditCheckProvider' and 'OBJ_AfterUpdateCreditCheckProvider'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateCreditCheckProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CreditCheckProviderID - creditcheckprovider id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateCreditCheckProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>CreditCheckProviderID - creditcheckprovider id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateCreditCheckProvider', {'Object'=>$self, 'Values'=> $hValues, 'CreditCheckProviderID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateCreditCheckProvider',  {'Object'=>$self, 'Values'=> $hValues, 'CreditCheckProviderID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::CreditCheck::API::Object::CreditCheckProvider->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'ConfigClass' => LoadClassByAlias('CreditCheckProviderConfig'),
        'Logging' => 0,
    };
}

#========================================================================================
# function     isDeprecated
# state        public
#----------------------------------------------------------------------------------------
# syntax       isDeprecated( $Shop );
#----------------------------------------------------------------------------------------
# description  returns if this credit check provider is deprecated
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | true if this credit check provider can be used | boolean
#========================================================================================
sub isDeprecated {
    my $self = shift;

    my $NowDateTime = GetCurrentDateTime();
    my $DeprecationDate = $self->get('DeprecationDate');

    return 0 unless isValid($DeprecationDate);
    return 0 if( Compare('DateTime', $NowDateTime, $DeprecationDate) < 0 );
    return 1;
}

1;
