#========================================================================================
# package      DE_EPAGES::DocdataPay::t::BuildDocdataPayRequest
# base         base DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for BuildDocdataPayRequest in UI/Shop
#========================================================================================
package DE_EPAGES::DocdataPay::t::BuildDocdataPayRequest;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::DocdataPay::UI::Shop;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use Test::More;
use Test::Exception;
use Data::Dumper;
use File::Spec;
use strict;

my $ROOTPATH = '/Test';


#========================================================================================
sub run_tests {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );

            eval {
                my ($continue, $hObjects) = testImport();
                return unless ( $continue );

                $DE_EPAGES::Object::API::Language::LANGUAGEID = 2;

                test_DocdataPayRequest_BuildWSDLWithOperation_positive('create');
                test_DocdataPayRequest_BuildWSDLWithOperation_positive('capture');
                test_DocdataPayRequest_BuildWSDLWithOperation_positive('status');

                test_DocdataPayRequest_BuildWSDLInvalidOperation_negative();
                test_DocdataPayRequest_BuildWSDLInvalidWSDLLocation_negative();

                test_DocdataPayRequest_CheckTotalNetAmount($hObjects);
                test_DocdataPayRequest_CaptureRequest_positive($hObjects);
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code());
                Test::More::diag($Error->longMessage());
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            ok( Cleanup(), 'cleanup');
        }
    );
}

#========================================================================================
sub testImport {
    my $testImportOK = 0;

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShop');

    my $PaymentMethodDocdataPay     = $Shop->path('PaymentMethods/DocdataPay');
    my $Order                       = $Shop->path('Customers/1001/Orders/1001');
    my $Basket                      = $Shop->path('Users/TestUser/Baskets/20491');
    my $LineItemContainer           = $Shop->path('Users/TestUser/Baskets/20491/LineItemContainer');
    my $LineItemPaymentDocdataPay   = $Shop->path('Users/TestUser/Baskets/20491/LineItemContainer/LineItemPaymentDocdataPay_50FFE7A2-8FF8-9E24-95C6-AC1532012696');
    my $LineItemProduct             = $Shop->path('Users/TestUser/Baskets/20491/LineItemContainer/LineItemProduct_50FFE7A1-5D7F-3387-F1BF-AC15320126B7');
    my $LineItemShipping            = $Shop->path('Users/TestUser/Baskets/20491/LineItemContainer/LineItemShipping_51CDB8F0-CA46-565A-D4A9-D5809AA5E458');

    $testImportOK = isa_ok($Shop,                       'DE_EPAGES::Shop::API::Object::Shop');
    $testImportOK = isa_ok($PaymentMethodDocdataPay,    'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay')      && $testImportOK;
    $testImportOK = isa_ok($Order,                      'DE_EPAGES::Order::API::Object::CustomerOrder' )                    && $testImportOK;
    $testImportOK = isa_ok($Basket,                     'DE_EPAGES::Order::API::Object::Basket' )                           && $testImportOK;
    $testImportOK = isa_ok($LineItemContainer,          'DE_EPAGES::Order::API::Object::LineItemContainer' )                && $testImportOK;
    $testImportOK = isa_ok($LineItemPaymentDocdataPay,  'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay' )   && $testImportOK;
    $testImportOK = isa_ok($LineItemProduct,            'DE_EPAGES::Product::API::Object::LineItemProduct' )                && $testImportOK;
    $testImportOK = isa_ok($LineItemShipping,           'DE_EPAGES::Order::API::Object::LineItemShipping' )                 && $testImportOK;

    return ($testImportOK, {
        'Shop'                      => $Shop,
        'PaymentMethodDocdataPay'   => $PaymentMethodDocdataPay,
        'Order'                     => $Order,
        'Basket'                    => $Basket,
        'LineItemContainer'         => $LineItemContainer,
        'LineItemPaymentDocdataPay' => $LineItemPaymentDocdataPay,
        'LineItemProduct'           => $LineItemProduct,
        'LineItemShipping'          => $LineItemShipping
    });
}

#========================================================================================
sub test_DocdataPayRequest_CheckTotalNetAmount {
    my ($hObjects) = @_;
    my $SubTestName = ( caller(0) )[3];

    my $hObtainedRequest = _createRequest($hObjects);

    my $hObtainedTotalNetAmount = $hObtainedRequest->{'invoice'}->{'totalNetAmount'};
    my $hExpectedTotalNetAmount = _getExpectedTotalNetAmount();

    is_deeply( $hObtainedTotalNetAmount, $hExpectedTotalNetAmount,
        "$SubTestName - Created hash matches expected hash and returned the expected TotalNetAmount." )
        or Test::More::diag Dumper($hObtainedTotalNetAmount);

    my $aItems = $hObtainedRequest->{'invoice'}->{'item'};

    my $CalculatedTotalNetAmount = 0;
    foreach my $hItem (@$aItems) {
        my $NetAmount = $hItem->{'netAmount'}->{'_'};

        $CalculatedTotalNetAmount = $CalculatedTotalNetAmount + $NetAmount;
    }

    is($CalculatedTotalNetAmount, $hExpectedTotalNetAmount->{'_'}, "$SubTestName - Calculated net amount matches the expected TotalNetAmount.");
}

sub _createRequest {
    my ($hObjects) = @_;

    my $Basket = $hObjects->{'Basket'};

    my $CreatedRequest = DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest->new();

    return $CreatedRequest->buildCreateRequest($Basket);
}

sub _getExpectedTotalNetAmount {

    my $hExpectedTotalNetAmount = {
        'currency' => 'EUR',
        '_'        => '1321'
    };

    return $hExpectedTotalNetAmount;
}

#========================================================================================
sub test_DocdataPayRequest_CaptureRequest_positive {
    my ($hObjects) = @_;
    my $SubTestName = ( caller(0) )[3];

    my $LineItemPaymentDocdataPay = $hObjects->{'LineItemPaymentDocdataPay'};

    my $CapturedRequest = DE_EPAGES::DocdataPay::API::SOAP::Request::CaptureRequest->new();

    my $hObtainedRequest = $CapturedRequest->buildCaptureRequest($LineItemPaymentDocdataPay);
    my $hExpectedRequest = _getExpectedCapturedRequest();

    is_deeply( $hObtainedRequest, $hExpectedRequest, "$SubTestName - Capture request message." )
        or Test::More::diag Dumper($hObtainedRequest);
}

sub _getExpectedCapturedRequest {

    my $hExpectedRequest = {
        'version'  => '1.2',
        'merchant' => {
            'name'     => 'epagesorderapi3_nw',
            'password' => 'ppkn8D5z'
        },
        'paymentId' => '4907435245',
        'amount'    => {
            'currency' => 'EUR',
            '_'        => '1572'
        }
    };

    return $hExpectedRequest;
}

#========================================================================================
sub test_DocdataPayRequest_BuildWSDLWithOperation_positive {
    my ($operation) = @_;
    my $SubTestName = (caller(0))[3];

    my $IsTest = 1;
    my $CompiledClient = DE_EPAGES::DocdataPay::API::SOAP::CompiledClient->new($IsTest);

    my $WebserviceCall = $CompiledClient->buildWSDL( $operation );

    is( ref($WebserviceCall), 'CODE', "$SubTestName - buildWSDL returns a function pointer when using '" . $operation . "' operation." );
}

#========================================================================================
# AM filters tests output and searches for "error" strings.
# This test is expected to fail, so we need to make sure
# that the standard error output  is redirected to /dev/null,
# to avoid false alarms like this:
# error: no operation `undefined' for portType
# {http://www.docdatapayments.com/services/paymentservice/1_2/paymentservice.wsdl}
# PaymentServiceSOAP, pick from
#========================================================================================
sub test_DocdataPayRequest_BuildWSDLInvalidOperation_negative {
    my $SubTestName = ( caller(0) )[3];

    my $IsTest = 1;
    my $CompiledClient = DE_EPAGES::DocdataPay::API::SOAP::CompiledClient->new($IsTest);

    throws_ok (
        sub {
            do {
                local *STDERR= '';
                open STDERR, '>', File::Spec->devnull() or die "Can't redirect STDERR to /dev/null: $!";
                $CompiledClient->buildWSDL( 'undefined' );
            }
        },
        qr/no operation \`undefined\' for portType/,
        "$SubTestName - Throws an exception if you try to use an invalid method."
    );
}

#========================================================================================
# AM filters tests output and searches for "error" strings.
# This test is expected to fail,
# that the standard error output is redirected to /dev/null,
# to avoid false alarms like this:
# error: file docdata_1_2.wsdl does not exist
#========================================================================================
sub test_DocdataPayRequest_BuildWSDLInvalidWSDLLocation_negative {
    my $SubTestName = ( caller(0) )[3];

    my $IsTest = 1;
    my $CompiledClient = DE_EPAGES::DocdataPay::API::SOAP::CompiledClient->new($IsTest);

    throws_ok (
        sub {
            do {
                local *STDERR = '';
                open STDERR, '>', File::Spec->devnull() or die "Can't redirect STDERR to /dev/null: $!";
                $CompiledClient->buildWSDL( 'create', '' );
            }
        },
        'DE_EPAGES::Core::API::Error',
        "$SubTestName - Throws an exception if WSDL location is invalid."
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete() if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('ImportOrder.xml', $ROOTPATH);
    return 1;
}

sub Main {
    run_tests();
    done_testing();
}

RunScript( 'Sub' => \&Main );
1;
