#========================================================================================
# package      DE_EPAGES::MultiStore::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::MultiStore::API::Cartridge;
use base DE_EPAGES::Design::API::DesignInstaller;

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (ExistsCartridgeByPackage);
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadObject LoadRootObject );
use DE_EPAGES::MultiStoreCommon::API::Table::Object;

use constant PACKAGE_SIZE => 1000;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::MultiStore::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::MultiStore.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
## no critic qw( ProhibitLongLines )
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/MultiStore',
        'Version'            => '7.81.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0','7.42.0','7.43.0','7.44.0','7.45.0','7.46.0','7.47.0','7.48.0','7.49.0','7.50.0','7.51.0','7.52.0','7.53.0','7.54.0','7.55.0','7.56.0','7.57.0','7.58.0','7.59.0','7.60.0','7.61.0','7.62.0','7.63.0','7.64.0','7.65.0','7.66.0','7.67.0','7.68.0','7.69.0','7.70.0','7.71.0','7.72.0','7.73.0','7.74.0','7.75.0','7.76.0','7.77.0','7.78.0','7.79.0','7.80.0'],
    );
    return bless $self, $class;
}
## use critic

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    if (ExistsCartridgeByPackage('DE_EPAGES::MultiStoreMaster')) {
        die 'You can not install MultiStore and MultiStoreMaster cartridge on the same database!';
    }
    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub patch_6_14_1 {
    my $self = shift;

    my $dbi = GetCurrentDBHandle();
    if($dbi->driverName() ne 'mysql'){
        die 'MultiStore is supported only for MySQL.';
    }

    #Remove obsolete objects..
    #Get all product of each CacheShop and delete their defaultsubproductid.
    GetLog()->info('Get all product of each CacheShop and delete their defaultsubproductid.');
    $dbi->do(<<'SQL');
update object o, attribute a, booleanattribute ba, shop s, product p, product pp
set pp.defaultsubproductid = NULL
where o.alias = 'IsCacheShop'
and o.objectid = a.attributeid
and a.attributeid = ba.attributeid
and ba.objectid = s.shopid
and p.shopid = s.shopid
and p.productid = pp.defaultsubproductid
SQL

    #Delete all CacheShops.
    GetLog()->info('Delete all CacheShops.');
    my $iShops = LoadRootObject->child('Shops')->childrenIterator;
    while( my $Shop = <$iShops> ) {
        $Shop->delete if $Shop->get('IsCacheShop');
    }

    $self->importPatchDir('Patch_6_14_1', 'Patch');
    return ;
}

# see EPG-25252
sub patchDBStructure_6_14_4 {
    my $self = shift;

    $self->importPatchDir('Patch_6_14_4', 'PatchDBStructure');

    my $dbi = GetCurrentDBHandle();

    unless( $dbi->existsTable('subshopobject') ) {

        $dbi->do( <<'END_SQL' );
CREATE TABLE subshopobject (
     contentobjectid      integer  not null
,    masterpath           varchar(1000)  not null

,   constraint pk_shubshopobjectmap             primary key (contentobjectid)
,   constraint fk_subshopobject_contentobjectid foreign key (contentobjectid)
        references contentobject                    (contentobjectid)
)
END_SQL
    }

    if( $dbi->existsTable('affiliateobjectmap') ) {

        my $Count = $dbi->execute('SELECT COUNT(*) FROM subshopobject')->[0]->[0];
        if( 0 == $Count ) {
            GetLog()->info('convert object paths with quotes from affiliateobjectmap to subshopobject');

            GetLog()->info('create temporary tables');
            $dbi->do( <<'END_SQL' );
CREATE TEMPORARY TABLE _opath (
    objectid INT NOT NULL PRIMARY KEY,
    shopid INT NOT NULL,
    path VARCHAR(1000),
    isnew BOOL,
    INDEX( path, shopid )
)
END_SQL

            $dbi->do( <<'END_SQL' );
CREATE TEMPORARY TABLE _children (
    objectid INTEGER NOT NULL PRIMARY KEY,
    shopid INT NOT NULL,
    path VARCHAR(1000) NOT NULL
)
END_SQL

            $dbi->dropProcedureIfExists('fill_opath');
            $dbi->do( <<'END_SQL' );
CREATE PROCEDURE fill_opath()
BEGIN

    INSERT INTO _opath (objectid, shopid, path, isnew)
    SELECT o.objectid, o.siteid, o.alias, 1
    FROM object o
    INNER JOIN shop s ON o.parentid = s.shopid
    WHERE o.alias IN( 'Products', 'Categories' );

    WHILE ROW_COUNT() > 0 DO

        INSERT INTO _children (objectid, shopid, path)
        SELECT o.objectid, o.siteid, CONCAT( op.path, '/', o.alias )
        FROM object o
        INNER JOIN _opath op ON o.parentid = op.objectid
        WHERE isnew = 1;

        UPDATE _opath SET isnew = 0;

        INSERT INTO _opath (objectid, shopid, path, isnew)
        SELECT objectid, shopid, path, 1
        FROM _children;

        DELETE FROM _children;

    END WHILE;
END
END_SQL

            GetLog()->info('create object paths of all categories and products');
            $dbi->do( 'CALL fill_opath()' );

            $dbi->transaction( sub {

                GetLog()->info('fill table subshopobject, set MasterPath');
                $dbi->do( <<'END_SQL' );
INSERT INTO subshopobject (contentobjectid, masterpath)
SELECT op.objectid, op.path
FROM _opath op
INNER JOIN affiliateobjectmap aom ON aom.objectpath = op.path AND aom.shopid = op.shopid;
END_SQL

                GetLog()->info('remove mapped paths from affiliateobjectmap');
                $dbi->do( <<'END_SQL' );
DELETE FROM affiliateobjectmap
WHERE objectpath IN (
    SELECT path
    FROM _opath
);
END_SQL
            });

            GetLog()->info('drop temporary tables');
            $dbi->dropProcedure('fill_opath');
            $dbi->dropTempTableIfExists('_children');
            $dbi->dropTempTableIfExists('_opath');
        }

    }

    return ;
}

sub patch_6_14_4 {
    my $self = shift;

    $self->importPatchDir('Patch_6_14_4', 'Patch');

    my $dbi = GetCurrentDBHandle();

    # see EPG-25252
    if( $dbi->existsTable('affiliateobjectmap') ) {
        GetLog()->info('convert remaining paths with quotes from affiliateobjectmap to subshopobject');
        $dbi->transaction( sub {
            my $aaRows = $dbi->execute('SELECT shopid, objectpath FROM affiliateobjectmap');
            foreach my $aRow ( @$aaRows ) {
                my ($ShopID, $Path) = @$aRow;
                LoadObject($ShopID)->path($Path)->set({'MasterPath'=>$Path});
            }
        });

        GetLog()->info('drop table affiliateobjectmap');
        $dbi->dropTable('affiliateobjectmap');
    }

    GetLog()->info('Rename product type SubShopProductClass to ProductTypes/MS_ProductClass.');
    $dbi->do(<<'SQL');
UPDATE object o_sspc
INNER JOIN shop s ON s.shopid = o_sspc.parentid AND o_sspc.alias = 'SubShopProductClass'
INNER JOIN object p_ptf ON s.shopid = p_ptf.parentid AND p_ptf.alias = 'ProductTypes'
SET o_sspc.parentid = p_ptf.objectid,
    o_sspc.alias = 'MS_ProductClass';
SQL

    return ;
}

sub patch_6_15_4 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();
    if($dbi->driverName() ne 'mysql'){
        die 'MultiStore is supported only for MySQL.';
    }

    $self->logInfo('-> Alter the structure of attributes qw(MS_WebServiceLogging MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword).');

    my $aSubShops = LoadRootObject()->child('Shops')->children;
    my %ShopAttributes;
    my $aAttributeNames = [ qw( MS_WebServiceLogging MS_WebServiceURL MS_WebServiceUser MS_WebServicePassword ) ];

    # 1. Get the old attribute values.
    foreach my $Shop ( @$aSubShops ){
        my $hAttributes = $Shop->get( [ 'MasterShop', @$aAttributeNames ] );
        my $MasterShop = delete $hAttributes->{'MasterShop'};
        $ShopAttributes{$Shop->alias} = { map { $_ => { $MasterShop => $hAttributes->{$_} } } @$aAttributeNames };
    }

    # 2. Update the attributes type.
    $self->importPatchDir('Patch_6_15_4', 'Patch');

    # 3. Set the new attributes values.
    foreach my $SubShop ( @$aSubShops ){
        $SubShop->set( $ShopAttributes{$SubShop->alias} );
    }
    return '6.16.0';
}

1;
