#==============================================================================
# §package      DE_EPAGES::PayPalPPCP::UI::BasketOrder
# §base         DE_EPAGES::Presentation::UI::Object;
# §state        private
#==============================================================================
package DE_EPAGES::PayPalPPCP::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Order::UI::CustomerOrder;
use DE_EPAGES::PayPalPPCP::API::Constants qw ( $ORDER_STATUS_CREATED $ORDER_STATUS_VOIDED $ORDER_STATUS_APPROVED $ORDER_STATUS_COMPLETED $ORDER_STATUS_PAYER_ACTION_REQUIRED );

#========================================================================================
# §function     ViewMultiPaymentPayPalPPCP
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->ViewMultiPaymentPayPalPPCP( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Subroutine for the payment page on checkout
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub ViewMultiPaymentPayPalPPCP {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrder = $Servlet->object;

    #-- Return if this basket has already been converted to an order
    return if $BasketOrder->instanceOf('Basket') && $BasketOrder->get('OrderGUID');

    # order process steps are created and hook for changing the process is triggered
    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    my $Container = $BasketOrder->container;
    my $PaymentLine = $Container->get('Payment');
    #-- Not currently operating on order objects
    if ( ! $PaymentLine->instanceOf('LineItemPaymentPayPalPPCP') || $BasketOrder->instanceOf('CustomerOrder') ) {
        #-- Take to the cart if basket, or show the order confirmation page if order
        $Servlet->redirect($BasketOrder->get('WebUrlUnitySSL'));
    }

    my $PayViaPUI = $PaymentLine->get('PayViaPUI');
    my $PaymentMethod = $PaymentLine->get('PaymentMethod');

    if ( $PayViaPUI ) {
        #-- Paying on invoice, why are we here? Redirect to basket / order
        $Servlet->redirect($BasketOrder->get('WebUrlUnitySSL'));
    }

    my $hOrderInfo = $PaymentLine->get('OrderInfoJSON');
    if ( $hOrderInfo && %$hOrderInfo ) {
        #-- Get an update to the ppcp order by asking PayPal for current information about it
        my $hJsonResponse = $PaymentMethod->getPPCPOrder($PaymentLine, 1);

        if ( $hJsonResponse ) {

            #-- This is a successful response
            if ( $hJsonResponse->{'id'}
                    && $hJsonResponse->{'status'}
                    && $hJsonResponse->{'intent'}
            ) {
                $PaymentLine->set({
                    'OrderInfoJSON' => $hJsonResponse,
                    'TransID2'      => $hJsonResponse->{'id'}
                });
            } else {
                #-- We have a json structure, presumably with errors
                if ( $hJsonResponse->{'name'} eq 'RESOURCE_NOT_FOUND' ) {
                    #-- Need to recreate this order, since it can no longer be found at PayPal
                    #-- Hopefully this condition never arises for 'completed' orders
                    $PaymentLine->set({
                        'OrderInfoJSON' => undef,
                        'OrderPayload'  => undef,
                        'TransID2'      => undef,
                    });
                }
            }

        }
    }

    #-- If for whatever reason, this order is 'completed', we need to convert the basket to an order, even if the
    # amounts differ, because the payment has already been captured . . .
    if ( _isPPCPOrderCompleted($PaymentLine) ) {
        my $Order = _doBasket2Order($Servlet);
        _redirectToOrderPage($Servlet, $Order);
    }

    $hOrderInfo = $PaymentLine->get('OrderInfoJSON');

    #-- Voided, recreate the order @ paypal
    #-- https://developer.paypal.com/api/orders/v2/#orders_create
    #-- OR
    #-- Scenario: Completely new basket, the 'express' button is clicked on the cart page, and then the window is immediately closed BEFORE paying.
    #--           What is important is that the order object at PayPal is created.
    #-- This order has 'application_context'=>'shipping_preference' set to 'GET_FROM_FILE', which means address selection is allowed at PayPal
    #-- When we go to patch this order, we can't seem to update this parameter, and will allow address selection. We don't want this because if
    #-- we get to this code in this subroutine, it will be via standard checkout and not express
    #-- Ergo, check if we are in this state and clear the order
    if ( $hOrderInfo && %$hOrderInfo ) {
        my $hOrderPayLoad = $PaymentLine->get('OrderPayload');
        if ( $hOrderInfo->{'status'} eq $ORDER_STATUS_VOIDED
            || ( ($hOrderInfo->{'status'} eq $ORDER_STATUS_CREATED || $hOrderInfo->{'status'} eq $ORDER_STATUS_PAYER_ACTION_REQUIRED) && $hOrderPayLoad->{'payment_source'}->{'paypal'}->{'experience_context'}->{'shipping_preference'} eq 'GET_FROM_FILE' ) ) {
            $PaymentLine->set({
                'OrderInfoJSON' => undef,
                'OrderPayload'  => undef,
                'TransID2'      => undef,
            });
            $hOrderInfo = undef;
        }
    }

    if ( ! $hOrderInfo || ! %$hOrderInfo ) {
        #-- If this call fails, OrderInfoJSON will be undefined and we show a message asking the customer to refresh the page, basically calling this function again
        my $hJsonResponse = $PaymentMethod->createPPCPOrder($PaymentLine);
        $PaymentLine->set({
            'OrderInfoJSON' => $hJsonResponse,
            'OrderPayload'  => defined $hJsonResponse ? $PaymentMethod->getOrderPayload($PaymentLine) : undef,
            'TransID2'      => $hJsonResponse->{'id'}
        }) if $hJsonResponse;

        return;
    }

    #-- At this point, we have order information from PayPal, AND it is not 'completed'

    #-- Is this order not valid anymore?
    #-- Patch it
    if ( ! _isPPCPOrderStillValid($PaymentLine) ) {

        my $Patched = $PaymentMethod->patchPPCPOrder($PaymentLine);
        if ( $Patched ) {
            #-- Even if we fail to retrieve updated order info in the next line, we can at least update the this attribute so that we don't try and patch again
            $PaymentLine->set({
                'OrderPayload' => $PaymentMethod->getOrderPayload($PaymentLine),
            });

            my $hJsonResponse = $PaymentMethod->getPPCPOrder($PaymentLine);
            $PaymentLine->set({
                'OrderInfoJSON' => $hJsonResponse,
                'TransID2'      => $hJsonResponse->{'id'},
            }) if $hJsonResponse;
        } else {
            #-- Show the payment page and ask the customer to refresh the page so that we can try again since we must patch in order to proceed
            $Servlet->vars('ApiCallFailed', 1);
            return;
        }
    }

    #-- At this point, our order should be current with the basket/order we are operating on
    #-- If the status is such that the customer has 'approved', we don't need to ask the customer to go through the process again, just capture it?
    $hOrderInfo = $PaymentLine->get('OrderInfoJSON');
    my $DoCaptureAnyway = 0;
    if ( $hOrderInfo->{'status'} eq $ORDER_STATUS_APPROVED ) {
        $DoCaptureAnyway = 1;
    }

    #-- Capture?
    my $CAP = $Servlet->form->value('CAP');
    if ( $CAP == 1 || $DoCaptureAnyway ) {
        my $hJsonResponse = $PaymentMethod->capturePPCPOrder($PaymentLine);
        # set the OrderInfoJSON and the PP order id for finding the order in an easy way (used for webhooks)
        $PaymentLine->set({
            'OrderInfoJSON' => $hJsonResponse,
            'TransID2'      => $hJsonResponse->{'id'},
        }) if $hJsonResponse;

        if ( _isPPCPOrderCompleted($PaymentLine) ) {
            my $Order = _doBasket2Order($Servlet);
            _redirectToOrderPage($Servlet, $Order);
        } else {
            #-- Do some error handling. Here, the order must at least be in 'APPROVED' status, or we wouldn't have tried to capture.
            $Servlet->vars('CaptureFailed', 1);
        }
    }
}

#=============================
#   Convert the basket to an order, if we are working on a basket object (it could be an order)
#=================================
sub _doBasket2Order {
    my ($Servlet) = @_;

    my $BasketOrder = $Servlet->object;
    my $PaymentLine = $BasketOrder->container->get('Payment');
    $PaymentLine->updateLineItemFromPPCP();

    if ( $BasketOrder->instanceOf('Basket') ) {
        $BasketOrder = DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $BasketOrder);
    }

    $BasketOrder->set({
        'PaidOn' => GetCurrentDateTime(),
    });

    return $BasketOrder;
}

#=============================
#   Show the Now order confirmation page
#=================================
sub _redirectToOrderPage {
    my ($Servlet, $Order) = @_;

    #-- This subroutine will do some misc. things and then redirect to the order
    $Servlet->vars('ObjectID', $Order->id);
    DE_EPAGES::Order::UI::CustomerOrder->ViewMultiCheckoutCustomerOrder($Servlet);
}

#=============================
#   Check to see if the order has the 'completed' status, which means a payment has been captured at PayPal
#=================================
sub _isPPCPOrderCompleted {
    my ($PaymentLine) = @_;

    return 0 unless $PaymentLine->get('OrderInfoJSON');
    return 1 if $PaymentLine->get('OrderInfoJSON')->{'status'} eq $ORDER_STATUS_COMPLETED;

    return 0;
}

#=============================
#   Check to see if the order information we sent to PayPal is still valid
#   Most importantly, the total amount
#=====================================
sub _isPPCPOrderStillValid {
    my ($PaymentLine) = @_;

    my $hOrderInfo = $PaymentLine->get('OrderInfoJSON');
    my $hOrderPayload = $PaymentLine->get('OrderPayload');

    return 0 if ! $hOrderInfo || ! $hOrderPayload;
    #-- Check if the totals, currency, etc. are still valid for this basket

    my $PaymentMethod = $PaymentLine->get('PaymentMethod');
    my $CurrentPayLoad = $PaymentMethod->getOrderPayload($PaymentLine);

    #-- Data::Dumper modifies variable types (e.g. integer/string)! Be careful with this as there was an obscure interaction with the JSON module:
    #-- { 'a' => 1 }, the value of 'a' is an integer, but after logging it with Data::Dumper, it becomes { 'a' => '1' }
    #-- Technically, this is still a 1, and perl could care less, but JSON does!
    #-- This is especially important since we are comparing a JSON string below, and '{"a":1}' is different to '{"a":"1"}'

    #-- Basically, DON'T log these variables before comparing them
    my $Json = JSON->new;
    if ( $Json->canonical->encode($hOrderPayload) eq $Json->canonical->encode($CurrentPayLoad) ) {
        return 1;
    }

    return 0;
}

1;
