#========================================================================================
# package      DE_EPAGES::Product::API::ProductPatch
# state        public
#----------------------------------------------------------------------------------------
# description  Implements JSON-PATCH (RFC 6902) for the product object.
#========================================================================================
package DE_EPAGES::Product::API::ProductPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw (trim);
use DE_EPAGES::Core::API::TestParameter qw (TestFloat TestArray);
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::DAL::UnitOfMeasurement qw ( DAL_GetInfoUnitOfMeasurementByAbbreviation );
use DE_EPAGES::Product::API::Product qw (TestUPCEAN);
use DE_EPAGES::Product::API::ProductImageService;
use DE_EPAGES::Shop::API::ShopTaxClass;
use DE_EPAGES::WebService::API::Util qw (ReadJSONBoolean);

use List::Util qw (reduce);
use feature 'switch';
use strict;

## no critic (ProhibitUnusedPrivateSubroutines)



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Product );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# input        $Product | A Product | DE_EPAGES::Product::API::Object::Product
# input        $LanguageId | A LanguageID | integer
# input        $CurrencyId | A CurrencyId | integer
# return       $self | self | DE_EPAGES::Product::API::ProductPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Product, $LanguageId, $CurrencyId) = @_;
    my $self  = bless {
        'Product' => $Product,
        'LanguageId' => $LanguageId,
        'CurrencyId' => $CurrencyId,
    }, $class;
    return $self;
}



sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};
    my $LanguageId = $self->{'LanguageId'};
    my $CurrencyId = $self->{'CurrencyId'};

    if($Path eq '/productNumber') {
        $Product->set({'Alias' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/name') {
        $Product->set({'Name' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/visible') {
        $Product->set({'IsVisible' => ReadJSONBoolean($Value)});
        return;
    }

    if($Path eq '/shortDescription') {
        $Product->set({'Description' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/description') {
        $Product->set({'Text' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/deliveryPeriod') {
        $Product->set({'DeliveryPeriod' => $Value});
        return;
    }

    if($Path eq '/manufacturer') {
        $Product->set({'Manufacturer' => $Value});
        return;
    }

    if($Path eq '/upc' || $Path eq '/ean') {
        TestUPCEAN('Value', $Value);
        $Product->set({'UPCEAN' => $Value});
        return;
    }

    if($Path eq '/essentialFeatures') {
        $Product->set({'MainCharacteristics' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/searchKeywords') {
        TestArray('Value', $Value);
        my $Keywords = $self->_convertKeywordsArrayToString($Value);
        $Product->set({'Keywords' => $Keywords}, $LanguageId);
        return;
    }

    if($Path eq '/priceInfo/price/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setListPrice($CurrencyId, $TaxModel, $Value);
        return;
    }

    if($Path eq '/priceInfo/manufacturerPrice/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setManufacturerPrice($CurrencyId, $TaxModel, $Value);
        return;
    }

    if($Path eq '/priceInfo/depositPrice/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setDeposit($CurrencyId, $TaxModel, $Value);
        return;
    }

    if($Path eq '/productImage') {
        my $ProductImageService = $self->_createProductImageService($Product);
        $ProductImageService->setSlideShowImageAsProductImage($Value);
        return;
    }

    if($Path eq '/stocklevel') {
        TestFloat('Value', $Value);
        my $CurrentStockLevel = $Product->get('StockLevel');
        if(not defined $CurrentStockLevel) {
            $CurrentStockLevel = 0;
        }
        my $NewStockLevel = $CurrentStockLevel + $Value;
        $Product->set({'StockLevel' => $NewStockLevel});
        return;
    }

    if($Path eq '/minStocklevel') {
        TestFloat('Value', $Value);
        $Product->set({'StockLevelAlert' => $Value});
        return;
    }

    if($Path eq '/manufacturerProductNumber') {
        $Product->set({'ManufacturerSKU' => $Value});
        return;
    }

    if($Path eq '/productLength') {
        TestFloat('Value', $Value);
        $Product->set({'Length' => $Value});
        return;
    }

    if($Path eq '/productWidth') {
        TestFloat('Value', $Value);
        $Product->set({'Width' => $Value});
        return;
    }

    if($Path eq '/productHeight') {
        TestFloat('Value', $Value);
        $Product->set({'Height' => $Value});
        return;
    }

    if($Path eq '/deliveryWeight') {
        return unless (
            defined($Value) &&
            defined($Value->{'amount'}) &&
            defined($Value->{'unit'})
        );
        my $aUnitOfMeasurementInfo = DAL_GetInfoUnitOfMeasurementByAbbreviation($Value->{'unit'});
        Error("delivery weight unit is not valid: $Value->{'unit'}")
            unless ( (scalar @{$aUnitOfMeasurementInfo} > 0) && ($aUnitOfMeasurementInfo->[0]->{'Type'} eq 'mass') );

        $Product->set({'Weight' => $Value->{'amount'}, 'WeightUnit' => LoadObject($aUnitOfMeasurementInfo->[0]->{'UnitID'}) });
        return;
    }

    Error("Adding in $Path is not implemented!");
}



sub _remove {
    my $self = shift;
    my ($Path) = @_;

    my $Product = $self->{'Product'};
    my $LanguageId = $self->{'LanguageId'};
    my $CurrencyId = $self->{'CurrencyId'};

    if($Path eq '/name') {
        $Product->set({'Name' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/shortDescription') {
        $Product->set({'Description' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/description') {
        $Product->set({'Text' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/deliveryPeriod') {
        $Product->set({'DeliveryPeriod' => undef});
        return;
    }

    if($Path eq '/manufacturer') {
        $Product->set({'Manufacturer' => undef});
        return;
    }

    if($Path eq '/upc' || $Path eq '/ean') {
        $Product->set({'UPCEAN' => undef});
        return;
    }

    if($Path eq '/essentialFeatures') {
        $Product->set({'MainCharacteristics' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/searchKeywords') {
        $Product->set({'Keywords' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/priceInfo/price/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setListPrice($CurrencyId, $TaxModel, undef);
        return;
    }

    if($Path eq '/priceInfo/manufacturerPrice/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setManufacturerPrice($CurrencyId, $TaxModel, undef);
        return;
    }

    if($Path eq '/priceInfo/depositPrice/amount') {
        my $Shop = $Product->getSite();
        my $TaxModel = $Shop->get('TaxModel');
        $Product->setDeposit($CurrencyId, $TaxModel, undef);
        return;
    }

    if($Path eq '/stocklevel') {
        $Product->set({'StockLevel' => undef});
        return;
    }

    if($Path eq '/minStocklevel') {
        $Product->set({'StockLevelAlert' => undef});
        return;
    }

    if($Path eq '/manufacturerProductNumber') {
        $Product->set({'ManufacturerSKU' => undef});
        return;
    }

    if($Path eq '/productLength') {
        $Product->set({'Length' => undef});
        return;
    }

    if($Path eq '/productWidth') {
        $Product->set({'Width' => undef});
        return;
    }

    if($Path eq '/productHeight') {
        $Product->set({'Height' => undef});
        return;
    }
    Error("Removing $Path is not implemented!");
}



sub _replace {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Product = $self->{'Product'};
    my $Shop = $Product->getSite();

    if($Path eq '/priceInfo/taxClass/taxClassId') {
        my $TaxClass = $self->_loadTaxClass($Value, $Shop);

        if(defined $TaxClass) {
            $self->_setTaxClass($TaxClass);
        } else {
            Error('TaxClass could not be found!');
        }

        return;
    }

    if($Path eq '/stocklevel') {
        TestFloat('Value', $Value);
        $Product->set({'StockLevel' => $Value});

        return;
    }

    Error("Replacing $Path is not implemented!");
}



sub _loadTaxClass {
    my $self = shift;
    my ($GUID, $Shop) = @_;

    my $ShopTaxClass = $self->_createShopTaxClass($Shop);
    my $TaxClass = $ShopTaxClass->loadTaxClassbyGUID($GUID);

    return $TaxClass;
}



sub _setTaxClass {
    my $self = shift;
    my ($TaxClass) = @_;

    my $Product = $self->{'Product'};

    if($TaxClass->instanceOf('TaxClass') || $TaxClass->instanceOf('TaxClassUS')) {
        $Product->set({'TaxClass' => $TaxClass});

        # Note: if there is a TaxClassDigital, the TaxClass of a product will be ignored!
        $Product->set({'TaxClassDigital' => undef});
    }

    if($TaxClass->instanceOf('TaxClassDigital')) {
        $Product->set({'TaxClassDigital' => $TaxClass});
    }

    return;
}



sub _parseBoolean {
    my $self = shift;
    my ($Boolean) = @_;
    given( $Boolean ) {
        when('true') {
            return 1;
        }
        when ('false') {
            return 0;
        }
        default {
            Error("Could not parse Boolean $Boolean");
        }
    }
}



sub _convertKeywordsArrayToString {
    my $self = shift;
    my($aKeywords) = @_;

    my @TrimmedKeywords = map {trim($_)} @$aKeywords; # remove leading and trailing spaces
    my @Keywords = grep {length($_) > 0} @TrimmedKeywords; # remove empty strings
    my $KeywordString = reduce {"$a, $b"} @Keywords; # create string with comma-separated list of keywords

    return $KeywordString;
}



sub _createProductImageService {
    my $self = shift;
    my ($Product) = @_;
    return DE_EPAGES::Product::API::ProductImageService->new($Product);
}



sub _createShopTaxClass {
    my $self = shift;
    my ($Shop) = @_;
    return DE_EPAGES::Shop::API::ShopTaxClass->new($Shop);
}

1;
