use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject LoadObjectByPath LoadRootObject );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# §function         PatchUnityIsVATInfoVisible
# §state            public
#----------------------------------------------------------------------------------------
# §syntax           PatchUnityIsVATInfoVisible();
#----------------------------------------------------------------------------------------
# §description      Sets the value of IsVATInfoVisible to 0 for products with taxclass 'none'
#                   this on done only for Now shops and without adapted tax matrix
#========================================================================================
sub PatchUnityIsVATInfoVisible {
    my ($ShopAlias) = @_;

    my $SystemTaxMatrixID = LoadRootObject()->get('TaxMatrix')->id;
    if (defined $ShopAlias) {
        # single shop run
        my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);
        if ($Shop->isUnity()) {
            my $ShopTaxMatrix = $Shop->get('TaxMatrix');
            unless ($SystemTaxMatrixID eq $ShopTaxMatrix->id){
                warn('Patch of the IsVATInfoVisible failed for shop ID ' . $Shop->id . " (custom TaxMatrix found)");
            }
            # Unity shop
            _PatchUnityIsVATInfoVisible($Shop, $ShopTaxMatrix);
        } else {
            # non Unity shop
           print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
        }
    } else {
        # all Unity shops
        my $aShopIDs = GetUnityShopIDs();

        foreach my $ShopID (@$aShopIDs) {
            eval {
                my $Shop = LoadObject($ShopID);
                my $ShopTaxMatrix = $Shop->get('TaxMatrix');
                unless ($SystemTaxMatrixID eq $ShopTaxMatrix->id){
                    warn('Patch of the IsVATInfoVisible failed for shop ID ' . $ShopID . " (custom TaxMatrix found)");
                    return;
                }
                _PatchUnityIsVATInfoVisible($Shop, $ShopTaxMatrix);
            };
            warn('Patch of the IsVATInfoVisible failed for shop ID ' . $ShopID . " with error \"" . GetError() . "\".")
                if ExistsError();
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub _PatchUnityIsVATInfoVisible {
    my ($Shop, $ShopTaxMatrix) = @_;

    #find tax class none for the shop
    my $TaxClass =  $ShopTaxMatrix->getTaxClassByAlias('none');
    my $ProductClass = LoadClassByAlias('Product');
    # find products with tax class none and set attribute
    LoadObject( $_ )->set({'IsVATInfoVisible' => 0 })
        foreach @{$ProductClass->findObjects('ByTaxClassIDShopID', [$TaxClass->id, $Shop->id])};

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopid=s'    => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchUnityIsVATInfoVisible($ShopAlias); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Sets the value of IsVATInfoVisible for existing products with TaxClass 'none' on each unity shop
Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                      (optional)
                (if not given, the script is preformed for all Unity shops)
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );