#========================================================================================
# package      DE_EPAGES::ServiRed::UI::Shop
# state        public
# base         DE_EPAGES::Presentation::UI::Object
# description  handle ServiRed callbacks
#========================================================================================
package DE_EPAGES::ServiRed::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

## no critic qw(ProhibitLongLines RequireExtendedFormatting)
use strict;

use DateTime;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject );
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::ServiRed::API::Constants qw (
    %SERVIRED_ERRORCODES
    STATUS_NEW
    STATUS_AUTHORISED
    STATUS_CANCELLED
    STATUS_CAPTURED
    STATUS_DECLINED
);
use DE_EPAGES::ServiRed::API::Payment qw ( CalculateHash );
use DE_EPAGES::ServiRed::API::RedSysCallbackParams;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

# the return parameter names
my @RETURN_FIELDS = qw (
  Ds_Date
  Ds_Hour
  Ds_Amount
  Ds_Currency
  Ds_Order
  Ds_MerchantCode
  Ds_Terminal
  Ds_Response
  Ds_MerchantData
  Ds_SecurePayment
  Ds_TransactionType
  Ds_Card_Country
  Ds_AuthorisationCode
  Ds_ConsumerLanguage
  LineItemContainerID
  Ds_SignatureVersion
  Ds_MerchantParameters
  Ds_Signature
);

#========================================================================================

sub ErrorDefinition {
    return {
        'ServiRedNoCallBackParameter' =>
          'Function: -CallbackServiRedReturn- called from payment gateway without Parameters: ShopID (#ShopID)',
        'ServiRedCouldNotIdentOrder' =>
          'Function: -CallbackServiRedReturn- could not identify the Order (empty URL parameters) for Shop #ShopPath (#ShopID)',
        'ServiRedError' =>
          'Error (Code: #ResponseCode) received from payment gateway (Shop: #ShopPath (#ShopID), TransID: #TransID, LineItemContainer: #ContainerID)',
        'ServiRedSignatureMismatch' =>
          'Signature: #Signature does not match the calculated Signature: #Mac (Shop: #ShopPath (#ShopID), TransID: #TransID, LineItemContainer: #ContainerID'
    };
}

#========================================================================================
# name         ServiRedReturn
# state        public
#----------------------------------------------------------------------------------------
# description  Called function when the bank send POST data. The order is created at
#               this point in background.
# return       nothing because this is a direct call from a system
#========================================================================================
sub ServiRedReturn {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop           = $Servlet->object();
    my $hFormValues    = $Servlet->params();

    my %ServiRedValues = map { $_ => $hFormValues->{$_} } grep { ElementOf( $_, @RETURN_FIELDS ) } keys %$hFormValues;
    $self->_LogPayment('return parameters', \%ServiRedValues );

    $Servlet->vars( 'ViewObjectID', $Shop->id );
    $Servlet->vars( 'ViewAction',   undef );

    # test if we have callback parameters (SHA1 or SHA256)
    unless ( (defined $ServiRedValues{'Ds_MerchantParameters'}) || (defined $ServiRedValues{'Ds_Order'}) ) {
        $self->_LogPayment('Warning: ServiRedNoCallBackParameter', 'missing return parameters' );
        Error( 'ServiRedNoCallBackParameter', { 'ShopID' => $Shop->id, 'ShopPath' => $Shop->pathString } );
    }

    # load and test LineItemContainer object
    unless ( defined ( $ServiRedValues{'LineItemContainerID'} )
        && $self->_ExistsObject( $ServiRedValues{'LineItemContainerID'} ) )
    {
        $self->_LogPayment(
            'Error: no container',
            "no line item container with id $ServiRedValues{'LineItemContainerID'} exists"
        );
        Error(
            'NoLineItemContainerInShop',
            {
                'ShopID'              => $Shop->id,
                'ShopPath'            => $Shop->pathString,
                'LineItemContainerID' => $ServiRedValues{'LineItemContainerID'}
            }
        );
    }

    my $LineItemContainer = $Shop->loadObject( $ServiRedValues{'LineItemContainerID'}, 'LineItemContainer' );
    my $ParentObject = $LineItemContainer->parent;

    my $PaymentLineItem = $LineItemContainer->get('Payment');

    # check basket or order
    $self->_checkBasketOrder($ParentObject, $Shop);


    # check signature
    my $mac;
    if (defined $ServiRedValues{'Ds_MerchantParameters'}) {
        my $RedSysCallbackParams = $self->_getRedSysCallbackParamsOrError($PaymentLineItem, $Shop, $ServiRedValues{'Ds_MerchantParameters'});
        $self->_LogPayment('json return parameter', $RedSysCallbackParams->getJsonMerchantParameters() );
        my $hMerchantParameters = $RedSysCallbackParams->merchantParametersToHash();

        $self->_fillServiRedValues(\%ServiRedValues, $hMerchantParameters);
        $mac = $RedSysCallbackParams->getSignature();
    } else {
        my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
        $mac = CalculateHash(
            $ServiRedValues{'Ds_Amount'},   $ServiRedValues{'Ds_Order'},    $ServiRedValues{'Ds_MerchantCode'},
            $ServiRedValues{'Ds_Currency'}, $ServiRedValues{'Ds_Response'}, $PaymentMethod->get('SecretKey')
        );
    }



    # EPG-29217 prevent from continue on an already paid order because we get a second callback POST
    return if ( $self->_PostedRepeatedTransaction( $Shop, $PaymentLineItem, \%ServiRedValues ) );

    if ( $mac ne $ServiRedValues{'Ds_Signature'} ) {
        # only a warning because the order exists
        my %Vars = (
            'Signature'   => $ServiRedValues{'Ds_Signature'},
            'Mac'         => $mac,
            'TransID'     => $ServiRedValues{'Ds_Order'},
            'ContainerID' => $ServiRedValues{'LineItemContainerID'},
            'ShopID'      => $Shop->id,
            'ShopPath'    => $Shop->pathString,
        );
        $self->_LogPayment('Error: ServiRedSignatureMismatch', \%Vars );
        Error( 'ServiRedSignatureMismatch', \%Vars );
    }

    # update payment line item
    my %Update = (
        'ReturnCode' => $ServiRedValues{'Ds_Response'},
        'AuthCode'   => $ServiRedValues{'Ds_AuthorisationCode'},
        'IsSecure'   => $ServiRedValues{'Ds_SecurePayment'},
    );
    if ( defined ( $ServiRedValues{'Ds_Date'} ) && $ServiRedValues{'Ds_Date'} =~ m!^(\d\d)/(\d\d)/(\d\d\d\d)$! ) {
        my %date = ( 'day' => $1, 'month' => $2, 'year' => $3 );
        if ( defined ( $ServiRedValues{'Ds_Hour'} ) && $ServiRedValues{'Ds_Hour'} =~ /^(\d\d):(\d\d)$/ ) {
            $date{'hour'}   = $1;
            $date{'minute'} = $2;
        }
        $Update{'TransTime'} = DateTime->new(%date);
    }
    $PaymentLineItem->set( \%Update );

    # warn and return if we have an error code from the payment gateway
    my $IsTransactionOK
      = ( $SERVIRED_ERRORCODES{ $ServiRedValues{'Ds_Response'} } eq 'TRANSACTION_SUCCESSFUL' ) ? 1 : 0;
    if ( !$IsTransactionOK ) {
        $PaymentLineItem->set( { 'TransStatus' => STATUS_DECLINED } );
        my %Vars = (
            'ResponseCode' => $ServiRedValues{'Ds_Response'},
            'TransID'      => $ServiRedValues{'Ds_Order'},
            'ContainerID'  => $ServiRedValues{'LineItemContainerID'},
            'ShopID'       => $Shop->id,
            'ShopPath'     => $Shop->pathString,
        );
        $self->_LogPayment('Warning: ServiRedPaymentDeclined', \%Vars );

        #EPG-22338 remove warning
        #Warning('ServiRedPaymentDeclined', \%Vars);

        # reject order if exists (order was created before payment)
        if ( $ParentObject->instanceOf('CustomerOrder') ) {
            $ParentObject->set( { 'CancelledOn' => $self->_GetCurrentDBHandle()->currentDateTime } );
        }

        if ($Shop->get('IsUnityRegistered')) {
            $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'});
            $Servlet->redirect($ParentObject->get('WebUrlUnitySSL'));
            return;
        }
        return;
    }

    $self->_LogPayment('Success', "payment successful for container $ServiRedValues{'LineItemContainerID'}" );
    my $TransStatus;
    if ( defined $ServiRedValues{'Ds_TransactionType'} ) {
        if ( $ServiRedValues{'Ds_TransactionType'} eq '0' ) {
            $TransStatus = STATUS_CAPTURED;
        }
        elsif ( $ServiRedValues{'Ds_TransactionType'} eq '1' ) {
            $TransStatus = STATUS_AUTHORISED;
        }
    }
    if ( !defined ($TransStatus) ) {
        $TransStatus = $PaymentLineItem->get('TransactionType') == 0 ? STATUS_CAPTURED : STATUS_AUTHORISED;
    }
    $PaymentLineItem->set( { 'TransStatus' => $TransStatus } );

    # create the Order only if we have checked all parameters....
    #-- if the Basket object still exist
    my $Order
      = $ParentObject->instanceOf('CustomerOrder')
      ? $ParentObject
      : DE_EPAGES::Order::UI::Basket->doBasket2Order( $Servlet, $ParentObject );

    $Order->set( { 'PaidOn' => $self->_GetCurrentDBHandle()->currentDateTime } ) if ( $TransStatus eq STATUS_CAPTURED );

    return;
}

#========================================================================================
# name         CallbackServiRedReturn
# state        public
#----------------------------------------------------------------------------------------
# description  Callback function when a user clicks return to merchant page from the
#               last step of payment. this function is only called if the payment process
#               was successfull
#========================================================================================
sub CallbackServiRedReturn {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop        = $Servlet->object();
    my $Form        = $Servlet->form();
    my $hFormValues = $Servlet->params();

    my %ServiRedValues = map { $_ => $hFormValues->{$_} } grep { ElementOf( $_, @RETURN_FIELDS ) } keys %$hFormValues;
    $self->_LogPayment('callback parameters', \%ServiRedValues );

    # test if we can use the LineItemContainerID from the URL to identify the order
    unless ( defined $ServiRedValues{'LineItemContainerID'} ) {
        $self->_LogPayment('Warning: ServiRedCouldNotIdentOrder', 'missing parameter LineItemContainerID' );
        Error( 'ServiRedCouldNotIdentOrder', { 'ShopID' => $Shop->id, 'ShopPath' => $Shop->pathString } );
    }

    unless ( $self->_ExistsObject( $ServiRedValues{'LineItemContainerID'} ) ) {
        $self->_LogPayment(
            'Error: ServiRed_RETURN_STAMP',
            "no line item container with id $ServiRedValues{'LineItemContainerID'} exists"
        );

        $Form->executeFormError( {
                'Name'       => 'ServiRed_RETURN_STAMP',
                'Reason'     => 'BadCallback_NoPaymentGUID',
                'ViewAction' => 'ViewPaymentServiRedReturn'
            }
        );

        if ($Shop->get('IsUnityRegistered')) {
            return;
        }
    }

    my $LineItemContainer = $Shop->loadObject( $ServiRedValues{'LineItemContainerID'}, 'LineItemContainer' );
    my $PaymentLineItem = $LineItemContainer->get('Payment');

    my $ParentObject = $LineItemContainer->parent();

    # test if we have DS callback parameters
    unless ( (defined $ServiRedValues{'Ds_MerchantParameters'}) || (defined $ServiRedValues{'Ds_Order'}) ) {
        $self->_LogPayment('Warning: ServiRedNoCallBackParameter', 'missing backlink parameters' );

        # Redirection to Unity or ePages6 error page
        if ($Shop->get('IsUnityRegistered')) {
            $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'}); # NoCallBackParameters
            $Servlet->redirect($ParentObject->get('WebUrlUnitySSL'));
            return;
        } else {
            $Form->executeFormError( {
                    'Name'       => 'ServiRedNoCallBackParameter',
                    'Reason'     => 'BadCallback_NoCallBackParameters',
                    'ViewAction' => 'ViewPaymentServiRedReturn'
                } );
        }
    }

    my $mac;
    if (defined $ServiRedValues{'Ds_MerchantParameters'}) {
        my $RedSysCallbackParams = $self->_getRedSysCallbackParamsOrError($PaymentLineItem, $Shop, $ServiRedValues{'Ds_MerchantParameters'});
        $self->_LogPayment('json return parameter', $RedSysCallbackParams->getJsonMerchantParameters() );
        my $hMerchantParameters = $RedSysCallbackParams->merchantParametersToHash();

        $self->_fillServiRedValues(\%ServiRedValues, $hMerchantParameters);
        $mac = $RedSysCallbackParams->getSignature();
    } else {
        my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
        $mac = CalculateHash(
            $ServiRedValues{'Ds_Amount'},   $ServiRedValues{'Ds_Order'},    $ServiRedValues{'Ds_MerchantCode'},
            $ServiRedValues{'Ds_Currency'}, $ServiRedValues{'Ds_Response'}, $PaymentMethod->get('SecretKey')
        );
    }

    # handling of ServiRedReturn may still run...
    my $i = 0;
    while ( !$ParentObject->instanceOf('CustomerOrder') && $i++ < 5 ) {
        sleep (1);
        $ParentObject = $LineItemContainer->parent();
    }

    # test if we have an order...
    if ( !$ParentObject->instanceOf('CustomerOrder') ) {
        # the order has not been created yet.
        # we probably missed the ServiRedReturn POST request
        $self->_LogPayment('Error: ServiRedNotAnOrder',
                {
                    'LineItemContainer' => $LineItemContainer->pathString,
                    'ContainerID'       => $ServiRedValues{'LineItemContainerID'},
                    'TransID'           => $ServiRedValues{'Ds_Order'},
                }
        );

        # Redirection to Unity or ePages6 error page
        if ($Shop->get('IsUnityRegistered')) {
            $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'}); # NoAsyncCallRecceived
            $Servlet->redirect($ParentObject->get('WebUrlUnitySSL'));
            return;
        } else {
            $self->_noAsynchronousCallReceived( $Servlet, \%ServiRedValues );
        }
    } else {
        my $LineItemPayment = $LineItemContainer->get('Payment');
        my $TransStatus = $LineItemPayment->get('TransStatus');

        if ( $TransStatus eq STATUS_NEW ) {
            $self->_LogPayment('Error: No Asynchronous POST received',
                    {
                        'LineItemContainer' => $LineItemContainer->pathString(),
                        'ContainerID'       => $ServiRedValues{'LineItemContainerID'},
                        'TransID'           => $ServiRedValues{'Ds_Order'},
                    }
            );

            # Redirection to Unity or ePages6 error page
            if ($Shop->get('IsUnityRegistered')) {
                $PaymentLineItem->set({'PaymentErrorCode' => 'Failed'}); # NoAsyncCallRecceived
                $Servlet->redirect($ParentObject->get('WebUrlUnitySSL'));
                return;
            } else {
                $self->_noAsynchronousCallReceived( $Servlet, \%ServiRedValues );
            }
        }
    }

    # check if parameters are sent back, and if so, check error codes
    if ( defined $ServiRedValues{'Ds_Response'} ) {
        my $IsTransactionOK
          = ( $SERVIRED_ERRORCODES{ $ServiRedValues{'Ds_Response'} } eq 'TRANSACTION_SUCCESSFUL' ) ? 1 : 0;
        # error if we have an error code from the payment gateway
        # this error can only happen if the payment have internal problems because
        # this sub normally is only called if the payment process is successful
        unless ($IsTransactionOK) {
            my %Vars = (
                'ResponseCode' => $ServiRedValues{'Ds_Response'},
                'TransID'      => $ServiRedValues{'Ds_Order'},
                'ContainerID'  => $ServiRedValues{'LineItemContainerID'},
                'ShopID'       => $Shop->id,
                'ShopPath'     => $Shop->pathString,
            );
            $self->_LogPayment('Error: Payment not successful', \%Vars );

            #EPG-22338 remove warning
            #Warning('ServiRedError', \%Vars);
        }

        # fraud test
        if ( $mac ne $ServiRedValues{'Ds_Signature'} ) {
            # only a warning because the order exists
            my %Vars = (
                'Signature'   => $ServiRedValues{'Ds_Signature'},
                'Mac'         => $mac,
                'TransID'     => $ServiRedValues{'Ds_Order'},
                'ContainerID' => $ServiRedValues{'LineItemContainerID'},
                'ShopID'      => $Shop->id,
                'ShopPath'    => $Shop->pathString,
            );
            $self->_LogPayment('Warning: ServiRedSignatureMismatch', \%Vars );

            #EPG-22338 remove warning
            #Warning('ServiRedSignatureMismatch', \%Vars);
        }
    }

    # yes we have an order
    my $Order = $ParentObject;
    $Servlet->vars( 'ViewObjectID', $Order->id );
    $Servlet->vars( 'ViewAction',   'View' );
    $Servlet->registerSession( $Order->get('User'), 0 );

    return;
}

#========================================================================================
# name         CallbackServiRedCancel
# state        public
#----------------------------------------------------------------------------------------
# description  Callback function, when the payment is cancelled.
#========================================================================================
sub CallbackServiRedCancel {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop        = $Servlet->object;
    my $hFormValues = $Servlet->params;

    my %ServiRedValues = map { $_ => $hFormValues->{$_} } grep { ElementOf( $_, @RETURN_FIELDS ) } keys %$hFormValues;
    $self->_LogPayment('return parameters', \%ServiRedValues );

    my $LineItemContainer = $Shop->loadObject( $ServiRedValues{'LineItemContainerID'}, 'LineItemContainer' );

    my $PaymentLineItem = $LineItemContainer->get('Payment');
    my $mac;
    if (defined $ServiRedValues{'Ds_MerchantParameters'}) {
        my $RedSysCallbackParams = $self->_getRedSysCallbackParamsOrError($PaymentLineItem, $Shop, $ServiRedValues{'Ds_MerchantParameters'});
        $self->_LogPayment('json return parameter', $RedSysCallbackParams->getJsonMerchantParameters() );
        my $hMerchantParameters = $RedSysCallbackParams->merchantParametersToHash();

        $self->_fillServiRedValues(\%ServiRedValues, $hMerchantParameters);
        $mac = $RedSysCallbackParams->getSignature();
    } else {
        my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
        $mac = CalculateHash(
            $ServiRedValues{'Ds_Amount'},   $ServiRedValues{'Ds_Order'},    $ServiRedValues{'Ds_MerchantCode'},
            $ServiRedValues{'Ds_Currency'}, $ServiRedValues{'Ds_Response'}, $PaymentMethod->get('SecretKey')
        );
    }

    # test if we have callback parameters
    unless ( defined $ServiRedValues{'Ds_Order'} ) {
        $self->_LogPayment('Warning: ServiRedNoCallBackParameter', 'missing parameter Ds_Order' );
        $Servlet->vars( 'ErrorCode', 'Unknown' ) ;
    }

    # load and test LineItemContainer object
    unless ( defined ( $ServiRedValues{'LineItemContainerID'} )
        && $self->_ExistsObject( $ServiRedValues{'LineItemContainerID'} ) )
    {
        $self->_LogPayment(
            'Error: no container',
            "no line item container with id $ServiRedValues{'LineItemContainerID'} exists"
        );
        Error(
            'NoLineItemContainerInShop',
            {
                'ShopID'              => $Shop->id,
                'ShopPath'            => $Shop->pathString,
                'LineItemContainerID' => $ServiRedValues{'LineItemContainerID'}
            }
        );
    }

    # check error codes
    if (   defined ( $ServiRedValues{'Ds_Response'} )
        && defined ( $SERVIRED_ERRORCODES{ $ServiRedValues{'Ds_Response'} } ) )
    {
        my $TransResult = $SERVIRED_ERRORCODES{ $ServiRedValues{'Ds_Response'} };
        $Servlet->vars( 'ErrorCode', $TransResult ) unless ( $TransResult eq 'TRANSACTION_SUCCESSFUL' );
    }
    else {
        $Servlet->vars( 'ErrorCode', 'Unknown' );
    }

    #  cancel order if exists (i.e. order was created before payment and ServiRedReturn was not called)
    # this can happen i.e. if the merchant id is wrong...

    # test if we have an order...
    my $ParentObject    = $LineItemContainer->parent();

    $PaymentLineItem->set( { 'TransStatus' => STATUS_CANCELLED } );

    if ( $ParentObject->instanceOf('CustomerOrder') ) {
        $ParentObject->set( { 'CancelledOn' => $self->_GetCurrentDBHandle()->currentDateTime } );
    } else {
        $PaymentLineItem->deleteLineItemPaymentStatusInfo();
        return $self->_redirectToPaymentStep($Servlet, $ParentObject);
    }

    $Servlet->vars( 'PaymentObject', $ParentObject->tleHash );

    if ($Shop->get('IsUnityRegistered')) {
        $PaymentLineItem->set({'PaymentErrorCode' => 'Canceled'});
        $Servlet->redirect($ParentObject->get('WebUrlUnitySSL'));
        return;
    } else {
        $Servlet->vars( 'ViewAction',    'ViewPaymentServiRedCancel' );
    }
    return;
}

sub _redirectToPaymentStep {
    my $self = shift;
    my ($Servlet, $Basket) = @_;

    my $Shop = $Basket->get('Shop');

    my $hParams = {
        'ErrorPaymentServiRed' => $Servlet->vars('ErrorCode'),
        'ViewAction' => 'View',
    };

    # in case of mobile or multistep we fallback to the default view action 'View'    
    if ($Shop->get('OrderProcess') and not $Servlet->vars('IsMobile')) {
        $hParams->{'ViewAction'} = 'ViewMultiCheckoutPayment'
    }

    my $hOptions = { 'Type' => 'sf/mobile', 'UseSSL' => 1, 'AddToken' => 1 };
    my $Url = $self->_BuildShopUrl($Basket, $hParams, $hOptions, $Servlet);

    $self->_LogPayment('CallbackServiRedCancel redirect to payment',{
        'ObjectID' => $Basket->id(),
        'ServletParams' => $hParams
    });

    $Servlet->redirect($Url);
    return;
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _PostedRepeatedTransaction {
    my $self = shift;
    my ( $Shop, $PaymentLineItem, $ServiRedValues ) = @_;

    if (
        defined ( $PaymentLineItem->get('TransStatus') )
        && (   $PaymentLineItem->get('TransStatus') eq STATUS_CAPTURED
            || $PaymentLineItem->get('TransStatus') eq STATUS_AUTHORISED )
        && defined ( $PaymentLineItem->get('TransTime') )
        && $SERVIRED_ERRORCODES{ $ServiRedValues->{'Ds_Response'} } eq 'TRANSACTION_REPEATED'
      )
    {
        my %Vars = (
            'ResponseCode' => $ServiRedValues->{'Ds_Response'},
            'TransID'      => $ServiRedValues->{'Ds_Order'},
            'ContainerID'  => $ServiRedValues->{'LineItemContainerID'},
            'ShopID'       => $Shop->id,
            'ShopPath'     => $Shop->pathString,
        );
        $self->_LogPayment('Warning: CallBack POST repeated over a successfull payment', \%Vars );
        return 1;
    }
    else {
        return 0;
    }
}

#===========================================================================
sub _getRedSysCallbackParamsOrError {
    my $self = shift;
    my ($PaymentLineItem, $Shop, $merchantParameters) = @_;

    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
    my $secretKey = $PaymentMethod->get('SecretKey');
    my $RedSysCallbackParams = DE_EPAGES::ServiRed::API::RedSysCallbackParams->new();
    eval {
        $RedSysCallbackParams->decryptCallbackParams($merchantParameters, $secretKey, $Shop);
    };
    if (ExistsError()) {
        Error( 'DecryptionError' );
    }
    return $RedSysCallbackParams;
}

#===========================================================================
sub _checkBasketOrder {
    my $self = shift;
    my ($ParentObject, $Shop) = @_;

    unless ( $ParentObject->instanceOf('Basket') || $ParentObject->instanceOf('CustomerOrder') ) {
        $self->_LogPayment('Error',
                sprintf ( 'invalid class of LineItemContainer->parent %d: %s',
                          $ParentObject->id,$ParentObject->class->alias )
        );
        Error(
            'InvalidClassOfLineItemContainer',
            {
                'ShopID'       => $Shop->id,
                'ShopPath'     => $Shop->pathString,
                    'LineItemContainerParentID'     => $ParentObject->id,
                    'LineItemContainerParentObject' => $ParentObject->class->alias
            }
        );
    }

    return;
}

#===========================================================================
sub _fillServiRedValues {
    my $self = shift;
    my ($hServiredValues, $hMerchantParameters) = @_;

    foreach my $value (@RETURN_FIELDS) {
        if (defined $hMerchantParameters->{$value}) {
            $hServiredValues->{$value} = $hMerchantParameters->{$value};
        }
    }
    return;
}

#===========================================================================
sub _noAsynchronousCallReceived {
    my $self = shift;
    my ($Servlet, $ServiRedValues) = @_;

    $Servlet->vars('LineItemContainerID', $ServiRedValues->{'LineItemContainerID'});
    $Servlet->vars('DSParams', [
            map {
                { 'Param' => $_, 'Value' => $ServiRedValues->{$_} }
            } (keys %{$ServiRedValues})
    ]);

    $Servlet->form->executeFormError( {
            'Name'       => 'OrderID',
            'Reason'     => 'NoAsyncCallReceived',
            'ViewAction' => 'ViewPaymentServiRedReturn'
        }
    );
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ($LogTitle, $DebugStuff) = @_;

    LogPayment('ServiRed', $LogTitle, $DebugStuff);

    return;
}

sub _ExistsObject {
    my $self = shift;
    my ($Object) = @_;

    return ExistsObject($Object);
}

sub _GetCurrentDBHandle {
    my $self = shift;

    return GetCurrentDBHandle();
}

sub _BuildShopUrl {
    my $self = shift;
    my ( $Object, $hParams, $hOptions, $Servlet ) = @_;
    return BuildShopUrl( $Object, $hParams, $hOptions, $Servlet );
}

1;
