#========================================================================================
# package      DE_EPAGES::TLE::API::Lexer
# state        private
#----------------------------------------------------------------------------------------
# description  Some constants used within the template compiling/executing process
#========================================================================================
package DE_EPAGES::TLE::API::Lexer;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    Lexer
);

use DE_EPAGES::TLE::API::LexConstants;
use DE_EPAGES::Core::API::Error qw ( Error );

use constant {
    CODE_LENGTH => 20 # number of characters of code in error messages
};


sub ErrorDefinition {
    return {
       TLE_LEXER_EXPRESSION => 'TLE Syntax error at line #line near \'#string\''
#       STRING_IN_EXPRESSION
#       STRING_NOT_CLOSED
    };
}

#==============================================================================
# function      Lexer
# state         private
#------------------------------------------------------------------------------
# syntax        $Lexer->Lexer($hCode);
#------------------------------------------------------------------------------
# description   Parses and transforms template files.
#------------------------------------------------------------------------------
# input         $hParam| template code | ref.hash
#------------------------------------------------------------------------------
# return        $hResult | lexer results | ref.hash
#==============================================================================
sub Lexer { ## no critic qw(ProhibitExcessComplexity)
    my ($hCode) = @_;

    my $template = \$hCode->{'text'};

    return (0, '') if ($$template eq '');

    my $token = $ERROR;
    my $parameter = undef;
    my $text = '';
    my $lineno_begin = $hCode->{'lineno'};
    my $lookahead = substr($$template, 0, 1);

    ## no critic qw(ProhibitCascadingIfElse)
    if ($lookahead eq '#') {
        $$template =~ s/^\#//x;
        $$template =~ s/^(CDATA)\s?([^\(])/$1("")$2/xs;

        if ($$template =~ s/^JSSCRIPT//x) {
            $token = $JSSCRIPT;
            $text = '';
        }
        elsif ($$template =~ s/^END_JSSCRIPT//x) {
            $token = $END_JSSCRIPT;
            $text = '';
        }
        elsif ($$template =~ m/^($BEGIN_BLOCKS)\s*\(/ or $$template =~ m/^($END_BLOCKS)\b/) {
            if ($$template =~ s/^($BLOCKS)\b//s) {
                my $Match = $1;
                $parameter = $Match;
                no strict 'refs';  ## no critic qw(ProhibitNoStrict)
                $token = ${$Match};
                use strict 'refs';
                $text .= '#'.$parameter;
            }
            if (defined $token) {
                if($token eq $CDATA){
                    $hCode->{'lexer_context_outer'} = $hCode->{'lexer_context'};
                    $hCode->{'chaining_outer'} = $hCode->{'chaining'};
                    $hCode->{'lexer_context'}--;
                    $hCode->{'chaining'} = undef;
                }
                elsif($token eq $ENDCDATA ){
                    $hCode->{'lexer_context'} = $hCode->{'lexer_context_outer'};
                    $hCode->{'chaining'} = $hCode->{'chaining_outer'};
                    $hCode->{'lexer_context_outer'}--;
                    $hCode->{'chaining_outer'} = undef;
                }
            }
            if ($parameter =~ s/^($BEGIN_BLOCKS)\b//) {
                # remove space between keyword and open bracket, for example #IF (#Var) ....
                $$template =~ s/^\s+//;
                $hCode->{'lexer_context'}++ unless $hCode->{'lexer_context'}; #EXPRESSION START
            }
        }
        elsif ($$template =~ s/^BEGIN(\W.*?)#END\b//s) {
            my $temp = $1;
            $parameter = $temp;
            $token = $PERLSCRIPT;
            $hCode->{'lineno'} += ($temp =~ tr/\n//);
            $text .= '#BEGIN'.$parameter.'#END';
        }
        elsif ($$template =~ s/^REM(.*?)#ENDREM\b//s) {
            my $temp = $1;
            $hCode->{'lineno'} += ($temp =~ tr/\n//);
            return Lexer($hCode); # ignore token find next
        }
        elsif ($$template =~ s/^(\w+(\.\w+)*)\b//x) {
            $parameter = $1;
            $token = substr($$template, 0, 1) eq '(' ? $FUNCTIONNAME : $VARIABLE;
            if($parameter =~ /\./ && $token == $FUNCTIONNAME ){
                my @Parts = split(/\./,$parameter);
                my $Call = pop @Parts;
                my $Text = substr($hCode->{'text'}, 1);
                my $Object = join('.', @Parts);
                $Text = ($Text =~ /^\s*\)/ ? '' : ', ').$Text;
                $hCode->{'text'} = '(#'.$Object.', "'.$Call.'"'.$Text;
                $parameter = 'CallGUIObjectMethod';
                $hCode->{'chaining'} = '#CallGUIObjectMethod(#'.$Object.',"';
                push @{$hCode->{'Objects'}}, $hCode->{'chaining'};
            }
            $text .= '#'.$parameter;
            $hCode->{'lexer_context'}++ if $token == $FUNCTIONNAME && !$hCode->{'lexer_context'}; #EXPRESSION START
        }
        elsif (not $hCode->{'lexer_context'}) { # unknown token (a color ?)
            $text = '#';
            $token = $BODY;
        }
        else {
            Error('STRING_IN_EXPRESSION', {
                'string' => $text.$lookahead.substr($$template,0,CODE_LENGTH),
                'line'=>$hCode->{'lineno'}
            });
        }
    }
    elsif ($lookahead eq '[' and ($hCode->{'lexer_last_token'} == $LOOP
                or $hCode->{'lexer_last_token'} == $VARIABLE
                or $hCode->{'lexer_last_token'} == $FUNCTION
                or $hCode->{'lexer_last_token'} == $BLOCK)
                and $$template =~ s/^\[([^\]]+)\]//xs
    ){
        $parameter = $1;
        $token = $FORMAT;
        $text .=  '['.$parameter.']';
    }
    elsif ($hCode->{'lexer_context'}) {
        if ($lookahead eq '"') {
            $$template =~ s/^\"//x;
            my ($found_quot, $found_string) = (1, 1);
            while ($found_quot and $found_string) {
                if ($$template =~ s/^(.*?)\"//s) {
                    $parameter .= $1;
                    if (substr($$template, 0, 1) eq '"') {
                        $$template =~ s/^\"//x;
                        $parameter .= '"';
                    }
                    else {
                        $found_quot = 0;
                    }
                }
                else {
                    $found_string = 0;
                }
            }
            Error('STRING_NOT_CLOSED', {
                'string' => $text . $lookahead . substr($$template, 0, CODE_LENGTH),
                'line'   => $hCode->{'lineno'}
            }) unless $found_string;

            $token = $STRCONSTANT;
            my $temp = $parameter;
            $temp =~ s/"/""/gs; #"
            $text .= '"'.$temp.'"';
        }
        elsif ($lookahead eq '{' && $$template =~ s/^(\{\()(.*?)(\)\})/#CDATA $2#ENDCDATA/xs) {
            $token = $SPACE;
            $text .= '';
        }
        elsif ($lookahead =~ /\d/ ) {
            if ($$template =~ s/^([\d\.]+)//s) {
                $parameter = $1 if defined $1;
            }
            $token = $NUMCONSTANT;
            $text .= $parameter;
        }
        elsif (grep {$lookahead eq $_} @SINGLES) {
            $$template = substr($$template, 1); # remove \d from template
            $token = ord($lookahead);
            $text =  $lookahead;
            if ($lookahead eq '(') {
                $hCode->{'lexer_context'}++;
            }
            elsif ($lookahead eq ')') {
                $hCode->{'lexer_context'}--;
                if ($hCode->{'lexer_context'} == 1) { # EXPRESSION ENDS
                    $hCode->{'lexer_context'} = 0;
                    ## no critic qw(ProhibitDeepNests)
                    if (defined $hCode->{'chaining'}) {
                        my $tempCode = $hCode->{'Objects'}->[-1];
                        if (!($$template =~ s/^\s*\.(\w+)\(\)/$tempCode$1")/xs || $$template =~ s/^\s*\.(\w+)\(/$tempCode$1",/xs) ) {
                            $hCode->{'chaining'} = undef;
                            pop @{$hCode->{'Objects'}};
                        }
                    }
                    ## use critic
                }
            }
        }
        elsif ($lookahead eq '<' or $lookahead eq '>') {
            $$template =~ s/^[<>]//x; # remove <> from template
            my $temp = substr($$template, 0, 1);
            if ($temp eq '=') {
                $$template =~ s/^=//x; # remove = from template
            } else {
                $temp = '';
            }
            $parameter = 'N'.($lookahead eq '<' ? 'L' : 'G').($temp eq '=' ? 'E' : 'T');
            no strict 'refs'; ## no critic qw(ProhibitNoStrict)
            $token = ${$parameter};
            $text .= $lookahead . $temp;
        }
        elsif (($lookahead eq '!' or $lookahead eq '=') and $$template =~ s/^(=|!)=//) {
            if ($1 eq '=') {
                $token = $NEQ;
                $text .= '==';
            } else {
                $token = $NNE;
                $text .= '!=';
            }
        }
        elsif ($lookahead eq '=' && $$template =~ s/^=/,/) {
            $token = $SPACE;
            $text .= '';
        }
        elsif ($$template =~ s/^(\s+)// ) { # spaces
            my $temp = $1;
            $token = $SPACE;
            $text  = $temp;
            $hCode->{'lineno'} += ($temp =~ tr/\n//);
        }
        elsif ($$template =~ s/^(OR|AND|NEQ|NNE|EQ|NE|NLT|NLE|NGT|NGE|IN|NOT)\b//x) {
            $parameter = $1;
            no strict 'refs'; ## no critic qw(ProhibitNoStrict)
            $token = ${$1};
            use strict 'refs';
            $text .= $parameter;
        }
        elsif ($$template =~ s/^(\w[\w\d]*([-_][\w\d]+)*)\b//x) {
            $token = $STRCONSTANT;
            $parameter .= $1  if defined $1;
            $text .= '"'.$parameter.'"';
        }
        else {
            Error('TLE_LEXER_EXPRESSION', {
                'string' => $text . substr($$template, 0, CODE_LENGTH),
                'line'   => $hCode->{'lineno'}
            });
        }
    }
    else { # find next token (# or <script)
        # EPG-11828: catch warning: Malformed UTF-8 character (unexpected continuation byte 0xbb,
        # with no preceding start byte) in pattern match (m//) at D:\epages\Cartridges/DE_EPAGES/TLE/API/Lexer.pm line 157.
        local $SIG{__WARN__} = sub { die $_[0] };
        if ($$template =~ /(\#|<script\b.*?>)/ixs) {
            my $Key = $1;
            my $Pos = index $$template, $Key;
            if ($Pos > 0) {
                $text = substr $$template, 0, $Pos, '';
                $token = $BODY;
                $hCode->{'lineno'} += ($text =~ tr/\n//);
            }
            elsif ($Key =~ /<script/i) {
                # Don't handle jquery template or self-closing script tags as javascript block
                if ($Key =~ /type\s*=\s*(["']).*?jquery.*?\1/i or $Key =~ /<script.*?\/>/isx) {
                    $token = $BODY;
                    $text = substr $$template, 0, 7, '';
                }
                # Javascript block
                else {
                    $token = $BODY;
                    $text = substr $$template, 0, 7;

                    $$template =~ s/^<script(\b.*?>)(.*?)(<\/script>)/$1#JSSCRIPT$2#END_JSSCRIPT$3/isx;

                    #-- We need to find a closing </script> tag, because if we don't, we will reparse this token ad infinitum
                    #-- We could also use the same code as the 'if' condition above if we don't want to 'ERROR' and allow this rogue
                    #-- '<script>' tag through, but I think it is better to enforce this with an 'ERROR'
                    #-- Use the code 'TLE_ERROR' as existing code checks for it specifically to avoid showing a NativeDie
                    if ( !defined $3 || lc($3) ne '</script>' ) {
                        Error('TLE_ERROR', {
                            'line'      => $hCode->{'lineno'},
                            'Message'   => "Can not find closing '$Key' tag (postion $Pos) at line $hCode->{'lineno'}",
                        });
                    }
                }
            }
            else {
                die "Invalid key '$Key' (postion $Pos) at line $hCode->{'lineno'}";
            }
        }
        else { # nothing found until end
            $token = $BODY;
            $text =  $$template;
            $$template = '';
            $hCode->{'lineno'} += ($text =~ tr/\n//);
        }
    }
    ## use critic

    die 'token undef' unless defined $token;
    $token = $SPACE if ($token == $ERROR and $$template eq '');
    Error('TLE_LEXER', {
        'string' => $text . $lookahead . substr($$template, 0, CODE_LENGTH),
        'line'   => $hCode->{'lineno'}
    }) if $token == $ERROR;

    $hCode->{'lexer_last_token'} = $token;
    my %result;
    $result{'tag'}          = $token;
    $result{'line_start'}   = $lineno_begin;
    $result{'line_end'}     = $hCode->{'lineno'};
    $result{'parameter'}    = $parameter;
    $result{'text'}         = $text;
#    LogDebug('Token', \%result);
#    LogDebug('Context', $hCode->{'lexer_context'});
    return ($token, \%result);
}

1;