#========================================================================================
# package      DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate
# state        public
# base         DE_EPAGES::Shop::API::Object::ShopObject
# description  object interface for TrustedShopsCertificate.
#========================================================================================
package DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogDebug LogError );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::TrustedShops::API::Constants qw (
    %CERT_STATUS
    DEFAULT_OFFSET
    DEFAULT_VARIANT
);
use DE_EPAGES::TrustedShops::API::RemoteTrustedShops qw (
    CheckCertificate
    CheckLogin
    GetProtectionItems
);
use DE_EPAGES::TrustedShops::API::Tools qw ( TrustedShopsLogContext );
use DE_EPAGES::TrustedShops::API::TrustBadgeCode;
use DE_EPAGES::XML::API::DateTimeFormatXML;

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $TrustedShopsCertificate->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'TrustedShopsCertificate' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $TrustedShopsCertificate->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TrustedShopsCertificate = DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate->insert($hInfo);
# example      my $Class = LoadClassByAlias('TrustedShopsCertificate');
#               my $TrustedShopsCertificate = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertTrustedShopsCertificate' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $TrustedShopsCertificate | new object | object
# hook         OBJ_InsertTrustedShopsCertificate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TrustedShopsCertificateID - ObjectID             - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    $self->set({'IsExpertModeEnabled' => '0'});
    $self->set({'TrustbadgeOffset' => DEFAULT_OFFSET});
    $self->set({'TrustbadgeVariant' => DEFAULT_VARIANT});

    my $TrustBadgeCode = DE_EPAGES::TrustedShops::API::TrustBadgeCode->new();
    TrustedShopsLogContext($self, sub {
        eval {
            my $GeneratedTrustBadgeCode = $TrustBadgeCode->GenerateTrustBadgeCode($self->get('Alias'));
            LogDebug('Generated TrustBadge Code', $GeneratedTrustBadgeCode);
            $self->set({'TrustbadgeCardCoding' => $GeneratedTrustBadgeCode});
        };
        if ( ExistsError() ) {
            LogError( GetError() );
        }
    });

    TriggerHook('OBJ_InsertTrustedShopsCertificate', {'Object' => $self, 'TrustedShopsCertificateID' => $self->id() });
    delete $hInfo->{$_} foreach qw ( CertificationLanguage IsExcellence CertificateStatus WebserviceUsername WebservicePassword );
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TrustedShopsCertificate->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteTrustedShopsCertificate' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteTrustedShopsCertificate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TrustedShopsCertificateID - ObjectID             - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteTrustedShopsCertificate', {'Object' => $self, 'TrustedShopsCertificateID' => $self->id });
    LoadClassByAlias('TrustedShopsProtectionItem')->deleteObjects('ByTrustedShopsCertificateID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $TrustedShopsCertificate->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateTrustedShopsCertificate' and 'OBJ_AfterUpdateTrustedShopsCertificate'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateTrustedShopsCertificate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TrustedShopsCertificateID - ObjectID             - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateTrustedShopsCertificate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>TrustedShopsCertificateID - ObjectID             - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateTrustedShopsCertificate', {'Object'=>$self, 'Values'=> $hValues, 'TrustedShopsCertificateID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateTrustedShopsCertificate',  {'Object'=>$self, 'Values'=> $hValues, 'TrustedShopsCertificateID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::TrustedShops::API::Object::TrustedShopsCertificate->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsExcellence' => 0,
        'CertificateStatus' => 0,
    };
}

#========================================================================================
# function     updateAll
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ret = $Certificate->updateAll;
#----------------------------------------------------------------------------------------
# description  updates the certificate's status and all the protection items
#----------------------------------------------------------------------------------------
# return       $ret | true if successful, 0 on failure | boolean
#========================================================================================
sub updateAll {
    my $self = shift;

    my $IsInTestMode = $self->getSite()->get('IsInternalTestShop');
    my $Shop = $self->getSite();
    # check login
    my $WsUser     = $self->get('WebserviceUsername');
    my $WsPassword = $self->get('WebservicePassword');
    if (defined($WsUser) && defined($WsPassword)) {
        my $ret = CheckLogin($self->alias, $WsUser, $WsPassword, $IsInTestMode);
        unless (defined($ret) && $ret == 1) {
            $self->set({
                'WebserviceUsername' => undef,
                'WebservicePassword' => undef,
                'CertificateStatus'  => $CERT_STATUS{'DISABLED'}
            });
            $self->child('ProtectionItems')->delete if ($self->existsChild('ProtectionItems'));
            return ({'code' => undef , 'message' => 'invalid username/password'});
        }
    }

    # get info
    my $hInfo = CheckCertificate( $self->alias, $Shop );
    return ({'code' => undef , 'message' => 'invalid certificate'}) unless defined $hInfo;

    # update
    $self->set({
        'CertificationLanguage' => $hInfo->{'certificationLanguage'},
        'IsExcellence'          => lc($hInfo->{'typeEnum'}) eq 'excellence'  ?  1 : 0,
        'CertificateStatus'     => $CERT_STATUS{$hInfo->{'stateEnum'}} // $CERT_STATUS{'OTHER'}
    });

    # remove protection items if certificate is invalid or a classic one
    if ($hInfo->{'stateEnum'} eq 'INVALID_TS_ID' ||
        $hInfo->{'stateEnum'} eq 'CANCELLED'     ||
        $hInfo->{'stateEnum'} eq 'DISABLED'      ||
        !$self->get('IsExcellence')
    ) {
        $self->child('ProtectionItems')->delete if $self->existsChild('ProtectionItems');
        return ({'code' => 1, 'message' => 'OK'});
    }
    else { # update protection items
        my $ret = $self->updateProtectionItems;

        if($ret == 1){
            return ({'code' => $ret,'message' => 'OK'});
        } else {
            return ({'code' => $ret,'message' => 'Error getting protection items'});
        }
    }
}

#========================================================================================
# function     updateProtectionItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ret = $Certificate->updateProtectionItems;
#----------------------------------------------------------------------------------------
# description  calls the webservice function getProtectionItems and updates the list
#               of the certificate's protection items
#----------------------------------------------------------------------------------------
# return       $ret | true if successful, 0 on failure | boolean
#========================================================================================
sub updateProtectionItems {
    my $self = shift;

    # get protection items from trusted shops
    my $ahProtectionItems = GetProtectionItems($self->alias, $self->getSite()->get('IsInternalTestShop'));
    return 0 unless defined $ahProtectionItems;

    # get existing protection items in our database
    my %ExistingProtectionItems = map { $_->alias => $_ } @{$self->get('ProtectionItems')};
    my $DateParser = DE_EPAGES::XML::API::DateTimeFormatXML->new;

    foreach my $hItem (@$ahProtectionItems) {
        my $CreationDate = $DateParser->parse_datetime($hItem->{'creationDate'});
        if (exists($ExistingProtectionItems{$hItem->{'tsProductID'}})) {
            # update
            my $ProtectionItem = delete($ExistingProtectionItems{$hItem->{'tsProductID'}});
            $ProtectionItem->set({
                'CurrencyID'      => $hItem->{'currency'},
                'ProtectedAmount' => $hItem->{'protectedAmountDecimal'},
                'NetFee'          => $hItem->{'netFee'},
                'CreationDate'    => $CreationDate
            });
        }
        else {
            # insert
            $self->folder('ProtectionItems')->insertChild(LoadClassByAlias('TrustedShopsProtectionItem'), {
                'Alias'                   => $hItem->{'tsProductID'},
                'TrustedShopsCertificate' => $self,
                'CurrencyID'              => $hItem->{'currency'},
                'ProtectedAmount'         => $hItem->{'protectedAmountDecimal'},
                'NetFee'                  => $hItem->{'netFee'},
                'CreationDate'            => $CreationDate
            });
        }
    }
    # remove protection items that exist in our database but not at trusted shops
    $_->delete for (values %ExistingProtectionItems);

    return 1;
}

1;
