# package      DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Trusted Shops certificate creation controller
#========================================================================================
package DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Object::API::Table::Language qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale qw ( GetInfoLocale );
use DE_EPAGES::TrustedShops::API::Constants qw ( %CERT_STATUS %IS_EXCELLENCE_CERTIFICATE );
use DE_EPAGES::TrustedShops::API::RemoteTrustedShops qw ( CheckCertificate );
use DE_EPAGES::Core::API::PerlTools qw ( castToBoolean );
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# input        $Shop | Shop
# input        $TrustedShopsID | TrustedShopsCertificate ID
# input        $IsExcellence | Excellence
# input        $CertificateLanguage | Language of certificate
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::TrustedShops::API::TrustedShopsCertificateHandler
#========================================================================================
sub new {
    my $class = shift;
    my ($Shop, $TrustedShopsID, $IsExcellence, $CertificateLanguage) = @_;
    my $self  =  bless {
        'Shop' => $Shop,
        'TrustedShopsID'    => $TrustedShopsID,
        'IsExcellence'      => $IsExcellence,
        'CertificateLanguage' => $CertificateLanguage,
    }, $class;
    return $self;
}

#========================================================================================
# function     createCertificateObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       createCertificateObject();
#----------------------------------------------------------------------------------------
# description  Trusted Shops certificate creation method
#----------------------------------------------------------------------------------------
# return       $CreatedTrustedShopsID | integer | Object ID for Trusted Shops Certificate
#========================================================================================
sub createCertificateObject {
    my $self = shift;

    my $Shop = $self->{'Shop'};
    my $TrustedShopsCertificatesFolder = $Shop->folder('TrustedShopsCertificates');
    my $TrustedShopsID = $self->{'TrustedShopsID'};
    my $CertificateLanguage = $self->{'CertificateLanguage'};
    $CertificateLanguage //= $self->_getDefaultCertificateLanguage();

    my $Status = $CERT_STATUS{'OTHER'};
    my $IsExcellenceCertificate = $IS_EXCELLENCE_CERTIFICATE{'NO'};

    if ($self->_doOnlineCheck()){
        my $hCertificateParameters = $self->_CheckCertificate($TrustedShopsID, $Shop);
        $self->_checkValidCertification($hCertificateParameters);
        $self->_checkCertificationAlreadyExists($Shop->get('TrustedShopsCertificates'), $hCertificateParameters);

        $CertificateLanguage = $hCertificateParameters->{'certificationLanguage'};
        $Status = $CERT_STATUS{$hCertificateParameters->{'stateEnum'}};
        $IsExcellenceCertificate = $IS_EXCELLENCE_CERTIFICATE{'YES'};
    }

    my $TrustedShopsCertificate = $self->_createTrustedShopsCertificate(
        $TrustedShopsCertificatesFolder,
        $TrustedShopsID,
        $CertificateLanguage,
        $Status,
        $IsExcellenceCertificate
    );

    return $TrustedShopsCertificate->id();
}
#----------------------------------------------------------------------------------------
sub _doOnlineCheck {
    my $self = shift;
    #Online check must happen if this is a ExcellenceCertificate!
    return castToBoolean($self->{'IsExcellence'});
}
#========================================================================================
# function     getCertificateObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCertificateObject();
#----------------------------------------------------------------------------------------
# description  Returns Trusted Shops certificate object
#----------------------------------------------------------------------------------------
# input        $TrustedShopsID | TrustedShopsCertificate ID
#----------------------------------------------------------------------------------------
# return       $TrustedShopsCertificate | object | Object for Trusted Shops Certificate
#========================================================================================
sub getCertificateObject {
    my $self = shift;
    my ($TrustedShopsID) = @_;

    return LoadObject($TrustedShopsID);
}

#----------------------------------------------------------------------------------------
sub _checkValidCertification {
    my $self = shift;
    my ($hCertificateParameters) = @_;

    Error('INVALID_CERTIFICATE') unless (
            $hCertificateParameters
            && $hCertificateParameters->{'certificationLanguage'}
            && $CERT_STATUS{$hCertificateParameters->{'stateEnum'}}
    );

    Error('NON_EXCELLENCE_CERTIFICATION') unless (lc($hCertificateParameters->{'typeEnum'}) eq 'excellence');

    return;
}
#----------------------------------------------------------------------------------------
sub _checkCertificationAlreadyExists {
    my $self = shift;
    my ($aExistingCertificates, $hCertificateParameters) = @_;
    foreach my $Certificate (@$aExistingCertificates) {
        if ($Certificate->get('CertificationLanguage') eq $hCertificateParameters->{'certificationLanguage'}) {
            Error('LANGUAGE_CERTIFICATE_EXISTS');
        }
    }

    return;
}
#----------------------------------------------------------------------------------------
sub _createTrustedShopsCertificate {
    my $self = shift;
    my ($TrustedShopsCertificatesFolder, $TrustedShopsID, $Language, $Status, $IsExcellence) = @_;

    my $TrustedShopsCertificateClass = $self->_LoadClassByAlias('TrustedShopsCertificate');
    my $TrustedShopObject = $TrustedShopsCertificatesFolder->insertChild($TrustedShopsCertificateClass, {
            'Alias'                 => $TrustedShopsID,
            'CertificationLanguage' => $Language,
            'IsExcellence'          => $IsExcellence,
            'CertificateStatus'     => $Status
            });

    return $TrustedShopObject;
}
#----------------------------------------------------------------------------------------
sub _CheckCertificate {
    my $self = shift;
    my ($TrustedShopsID, $Shop) = @_;
    return CheckCertificate($TrustedShopsID, $Shop);
}

#----------------------------------------------------------------------------------------
sub _getDefaultCertificateLanguage {
    my $self = shift;

    # TODO: Prevent adding more than a certificate with a certain language.
    # Optionally we could also disable adding certificates when the amount of
    # languages is <= than the amount of certificates.
    my $Shop = $self->{'Shop'};
    my $hInfoLocaleDefault = $self->_GetInfoLocale( $Shop->get('LocaleID') );
    my $hInfoLanguage = $self->_GetInfoLanguage( $hInfoLocaleDefault->{'LanguageID'} );
    return $hInfoLanguage->{'Code2'};
}
#----------------------------------------------------------------------------------------
sub _GetInfoLocale {
    my $self = shift;
    my ($LocaleID) = @_;
    return GetInfoLocale($LocaleID);
}
#----------------------------------------------------------------------------------------
sub _GetInfoLanguage {
    my $self = shift;
    my ($LanguageID) = @_;
    return GetInfoLanguage($LanguageID);
}
#----------------------------------------------------------------------------------------
sub _LoadClassByAlias {
    my $self = shift;
    my ($Alias) = @_;
    return LoadClassByAlias($Alias);
}
1;