#========================================================================================
# package      DE_EPAGES::Zettle::API::Constants
# state        private
#----------------------------------------------------------------------------------------
# description  constants for cartridge
#========================================================================================
package DE_EPAGES::Zettle::API::Constants;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    INTERNAL_CARTRIDGE_NAME
    ZETTLE_OBJECT_ALIAS
    ZETTLE_ENVIRONMENT_CLASS
    ALLOWED_EXPORT_INTERVALS
    APP_SCOPES
    API_PARTNER_ID
    API_MAX_NAME_LENGTH
    API_MAX_DESCRIPTION_LENGTH
    API_MAX_VARIATIONS_PER_PRODUCT
    API_MAX_PRODUCTS_PER_DAY
    API_IMAGE_FORMAT
    ZETTLE_BASIC_ATTRIBUTES
    EXPORT_STATUS
    VALIDATION_TOKEN_KEY
    ZETTLE_HOME_URL
    ZETTLE_ATTRIBUTES_PROFILE
    ZETTLE_IMAGE_PROFILE
    ZETTLE_PRICE_PROFILE
    WAIT_INTERVAL_AFTER_DELETE
    ZETTLE_TAXATION_TYPE_NONE
    ZETTLE_TAXATION_TYPE_VAT
    ZETTLE_TAXATION_TYPE_SALES_TAX
    ZETTLE_TAXATION_MODE_INCLUSIVE
    ZETTLE_TAXATION_MODE_EXCLUSIVE
    MAX_PRODUCTS_PER_EXPORT
    PRODUCT_EXPORT_WAIT_INTERVAL
    MAX_IMAGES_PER_EXPORT
    IMAGES_EXPORT_WAIT_INTERVAL
    EXPORT_TYPE
    DEFAULT_ZETTLE_WEBHOOKS
    MAX_DB_ERROR_TYPE_LENGTH
    MAX_DB_ERROR_MESSAGE_LENGTH
);

# This name is used to load objects from database (DON'T CHANGE!)
use constant INTERNAL_CARTRIDGE_NAME => 'Zettle';

# Name of the shop attribute which references the environment object (DON'T CHANGE!)
use constant ZETTLE_OBJECT_ALIAS => 'Zettle';

# Name of class that stores Zettle-related information (DON'T CHANGE!)
use constant ZETTLE_ENVIRONMENT_CLASS => 'ZettleEnvironment';

# Valid export intervals for 'Update interval' selection in Scheduled Export section
use constant ALLOWED_EXPORT_INTERVALS => [ 1, 3, 7, 30 ];

# See "iZettle Partner Integrations: Solution Documentation" page 27, section 4.3.3. Permission Scopes
# Scope parameter: https://github.com/iZettle/api-documentation/blob/master/authorization.adoc#ask-the-user-to-authorize-the-client
use constant APP_SCOPES => qw(
    READ:USERINFO
    READ:PRODUCT
    WRITE:PRODUCT
);

# Identifier provided by Simona Pappalardo from PayPal via email
use constant API_PARTNER_ID => 'epages';

use constant API_MAX_NAME_LENGTH => 256;
use constant API_MAX_DESCRIPTION_LENGTH => 1024;
# See DE_EPAGES/Zettle/Documents/API/ZettleAPI-MaxVariationsPerProduct.md
# See https://developer.zettle.com/docs/api/product-library/concepts/product-library-data
use constant API_MAX_VARIATIONS_PER_PRODUCT => 99;
# Maximum number of products to export in a day
# See https://developer.zettle.com/docs/api/product-library/user-guides/manage-products/create-products#bulk-import-of-products
use constant API_MAX_PRODUCTS_PER_DAY => 10000;
# Format of image to be exported to Zettle (must match ZETTLE_IMAGE_PROFILE)
# If there's a need to export a different format you must create the associated LastUpdateProfile
# By default, only 'ImageLarge' profile is available with regards to images
use constant API_IMAGE_FORMAT => 'ImageLarge';

# Basic product fields to monitor for changes in order to update 'ZettleAttributes' LastUpdateProfile.
# See build methods in DE_EPAGES::Zettle::API::WebService::DTO::DTOFactory to get a grasp on field parsing,
# and _UpdateProfiles() method in DE_EPAGES::Zettle::Hooks::Product.
use constant ZETTLE_BASIC_ATTRIBUTES => (
    qw (
        Name
        Description
        ManufacturerSKU
        UPCEAN
    )
    #TaxClass: not considered for 'ZettleAttributes' profile (automatically registered for price updates in 'ListPrice' profile)
    #ListPrice: not considered for 'ZettleAttributes' profile (automatically registered for price updates in 'ListPrice' profile)
);

#========================================================================================
# global       $EXPORT_STATUS | status of an export operation to Zettle
#               <ul>
#                   <li>FAILED - export finished with failure.
#                   <li>PENDING - export was not yet started.
#                   <li>RUNNING - export is running.
#                   <li>COMPLETED - export completed successfully.
#               </ul> | ref.hash
#========================================================================================
use constant EXPORT_STATUS => {
    'FAILED' => -1,
    'PENDING' => 0,
    'RUNNING' => 1,
    'COMPLETED' => 2,
};

use constant VALIDATION_TOKEN_KEY => 'ValidationToken';

use constant ZETTLE_HOME_URL => 'https://www.zettle.com';

# LastUpdateProfile used to keep track of changes in product attributes of interest for Zettle export
use constant ZETTLE_ATTRIBUTES_PROFILE => 'ZettleAttributes';
# LastUpdateProfile used to keep track of changes in product image for Zettle export
use constant ZETTLE_IMAGE_PROFILE => 'ImageLarge';
# LastUpdateProfile used to keep track of changes in product prices for Zettle export
use constant ZETTLE_PRICE_PROFILE => 'ListPrice';

# Zettle has a known bug when performing initial export after deleting products from Zettle. We need to wait the specified number of seconds (indicated by Zettle)
use constant WAIT_INTERVAL_AFTER_DELETE => 5;

# ZETTLE TAXATION TYPES: see DE_EPAGES/Zettle/Documents/API/ZettlePartnerDocumentation-Extracts.md, section 4.4.1.1. Taxation Types
# This constant indicates that Zettle merchant account doesn't apply VAT or sales tx to the product prices.
use constant ZETTLE_TAXATION_TYPE_NONE => 'NONE';
# This constant indicates that Zettle merchant account is configured to use country-specific VAT rates
use constant ZETTLE_TAXATION_TYPE_VAT => 'VAT';
# This constant indicates that Zettle merchant account is configured to use merchant-specific VAT rates
use constant ZETTLE_TAXATION_TYPE_SALES_TAX => 'SALES_TAX';
# This constant indicates that Zettle merchant account is configured to include tax in the prices
use constant ZETTLE_TAXATION_MODE_INCLUSIVE => 'INCLUSIVE';
# This constant indicates that Zettle merchant account is configured to exclude tax in the prices
use constant ZETTLE_TAXATION_MODE_EXCLUSIVE => 'EXCLUSIVE';
# Maximum number of products to include in an API product export call
# See https://developer.zettle.com/docs/api/product-library/concepts/product-library-concept#recommendations-and-limits
# CAUTION: This constant affects the number of entries registered in zettleproductexport DB table (do not decrease below 100)
use constant MAX_PRODUCTS_PER_EXPORT => 1000;
# Seconds to wait between each export request to product service
# NOTE: when in the need for sleep interval under one second refer to https://perldoc.perl.org/Time::HiRes#usleep-(-$useconds-)
use constant PRODUCT_EXPORT_WAIT_INTERVAL => 1;
# Maximun number of images to include in an API product images export call
use constant MAX_IMAGES_PER_EXPORT => 100;
# Seconds to wait between each export request to the image service
use constant IMAGES_EXPORT_WAIT_INTERVAL => 1;

#========================================================================================
# global       $EXPORT_TYPE | type of export
#               <ul>
#                   <li>INITIAL_EXPORT - Full product update (may include prices)
#                   <li>PRODUCT_UPDATE - Product incremental update (may include prices)
#                   <li>PRICE_UPDATE - Price incremental update. Only includes prices (no other product fields)
#               </ul> | ref.hash
#========================================================================================
use constant EXPORT_TYPE => {
    'INITIAL_EXPORT' => 1,
    'PRODUCT_UPDATE' => 2,
    'PRICE_UPDATE'   => 3,
};

# List of webhooks that we'll be suscribed to by default
use constant DEFAULT_ZETTLE_WEBHOOKS => [
    'InventoryBalanceChanged',
    'InventoryTrackingStarted',
    'InventoryTrackingStopped'
];

# Maximum length of a string to be stored in 'errortype' column of table zettleproductexport
use constant MAX_DB_ERROR_TYPE_LENGTH => 80;
# Maximum length of a string to be stored in 'errormessage' column of table zettleproductexport
use constant MAX_DB_ERROR_MESSAGE_LENGTH => 255;

1;
