#========================================================================================
# package      DE_EPAGES::Zettle::DAL::Zettle
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>zettledelete
#                   <li>zettleproductexport
#               </ul>
#========================================================================================
package DE_EPAGES::Zettle::DAL::Zettle;
use base qw (Exporter);

use strict;

our @EXPORT = qw (
    DAL_GetDeletedObjects
    DAL_ClearDeletedObjects
    DAL_SetZettleDeleted

    DAL_InsertZettleDelete
    DAL_UpdateZettleDelete
    DAL_DeleteZettleDelete
    DAL_GetAllPKeyZettleDeletes
    DAL_GetAllInfoZettleDeletes
    DAL_GetInfoZettleDelete
    DAL_GetPKeyZettleDeletesBySiteID
    DAL_GetInfoZettleDeletesBySiteID
    DAL_GetPKeyZettleDeletesByClassID
    DAL_GetInfoZettleDeletesByClassID
    DAL_InsertZettleProductExport
    DAL_UpdateZettleProductExport
    DAL_GetInfoZettleProductExport
    DAL_DeleteZettleProductExport
    DAL_GetInfoZettleProductExportByGlobalExportUUID
    DAL_GetInfoZettleProductExportBySiteID
    DAL_GetInfoZettleProductExportByStatus
);

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime UsePackage );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::DataCache::API::Cache qw ( GetCache UpdateCache GetCacheSize );

my $DAL_PACKAGE = 'DE_EPAGES::Database::API::DAL';
my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Zettle::DAL::'.$dbi->driverName.'::Zettle';
    UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

sub _DoFillInfoCache ($$) {
    my ($CacheName, $CountRows) = @_;
    my $dbi = GetCurrentDBHandle();
    my $CacheSize = GetCacheSize($dbi->cacheKey, $CacheName);
    return $CountRows < $CacheSize ? 1 : 0;
}

#========================================================================================
# function     DAL_GetDeletedObjects
#----------------------------------------------------------------------------------------
# syntax       $Iterator = DAL_GetDeletedObjects($SiteID, $ClassID, $DeletedAfter);
#----------------------------------------------------------------------------------------
# description  Returns deleted objects of a given class from a site.
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# input        $DeletedAfter | Date and time of the deletion on an object | datetime
#----------------------------------------------------------------------------------------
# return       $Iterator | iterator over hash references with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul>| iterator of hashes
#========================================================================================
sub DAL_GetDeletedObjects ($$$) {
    my ($SiteID, $ClassID, $DeletedAfter) = @_;
    return _CallDriver('GetDeletedObjects', $SiteID, $ClassID, $DeletedAfter);
}

#========================================================================================
# function     DAL_ClearDeletedObjects
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_ClearDeletedObjects($SiteID, $ClassID, $DeletedBefore);
#----------------------------------------------------------------------------------------
# description  Deletes information about objects in the site of a given type.
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
# input        $DeletedBefore | Date and time of the deletion on an object | datetime
#========================================================================================
sub DAL_ClearDeletedObjects ($$$) {
    my ($SiteID, $ClassID, $DeletedBefore) = @_;
    return _CallDriver('ClearDeletedObjects', $SiteID, $ClassID, $DeletedBefore);
}

#========================================================================================
# function     DAL_SetZettleDeleted
#----------------------------------------------------------------------------------------
# syntax       DAL_SetZettleDeleted($hParameter);
#----------------------------------------------------------------------------------------
# description  Adds an entry in table zettledelete with the current date.
#               Updates the date if an entry already exists.
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                  <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                  <li>SiteID        - Site ID              - int
#                  <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                  <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                  <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#               </ul> | ref.hash
#========================================================================================
sub DAL_SetZettleDeleted ($) {
    my ($hParameter) = @_;
    _CallDriver('SetZettleDeleted', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'AllZettleDelete');
        UpdateCache($dbi->cacheKey,'InfoZettleDelete', $hParameter->{'GUID'});
        UpdateCache($dbi->cacheKey,'ZettleDeleteBySiteID');
        UpdateCache($dbi->cacheKey,'ZettleDeleteByClassID', $hParameter->{'ClassID'});
    }
    return;
}

#========================================================================================
# function     DAL_InsertZettleDelete
#----------------------------------------------------------------------------------------
# syntax       DAL_InsertZettleDelete($hParameter);
#----------------------------------------------------------------------------------------
# description  insert entry in table zettledelete
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                  <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                  <li>SiteID        - Site ID              - int
#                  <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                  <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                  <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                  <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash
#========================================================================================
sub DAL_InsertZettleDelete ($) {
    my ($hParameter) = @_;
    _CallDriver('InsertZettleDelete', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'AllZettleDelete');
        UpdateCache($dbi->cacheKey,'InfoZettleDelete', $hParameter->{'GUID'});
        UpdateCache($dbi->cacheKey,'ZettleDeleteBySiteID');
        UpdateCache($dbi->cacheKey,'ZettleDeleteByClassID', $hParameter->{'ClassID'});
    }
    return;
}

#========================================================================================
# function     DAL_UpdateZettleDelete
#----------------------------------------------------------------------------------------
# syntax       DAL_UpdateZettleDelete($GUID, $hParameter);
#----------------------------------------------------------------------------------------
# description  updates an entry of table zettledelete
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
# input        $hParameter | with following keys:
#               <ul>
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash
#========================================================================================
sub DAL_UpdateZettleDelete ($$) {
    my ($GUID, $hParameter) = @_;
    my $hInfo = DAL_GetInfoZettleDelete($GUID);
    my $hChangedValues = {};
    my $dbi = GetCurrentDBHandle();
    foreach my $ChangedKey (qw ( SiteID ClassID PathFromSite ZettleUUID DeletedOn )) {
        next unless exists $hParameter->{$ChangedKey};
        next if not defined $hInfo->{$ChangedKey} and not defined $hParameter->{$ChangedKey};
        if ( defined $hParameter->{$ChangedKey}
             and not defined $hInfo->{$ChangedKey}
             or
             not defined $hParameter->{$ChangedKey}
             and defined $hInfo->{$ChangedKey}
        ) {
            $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey});
            next;
        }
        next if $hInfo->{$ChangedKey} eq $hParameter->{$ChangedKey};
        $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey})
    }
    return 0 unless scalar keys %$hChangedValues;

    _CallDriver('UpdateZettleDelete', $GUID, $hChangedValues);
    if($DO_CACHE) {
        UpdateCache($dbi->cacheKey,'InfoZettleDelete', $GUID);
        UpdateCache($dbi->cacheKey,'ZettleDeleteBySiteID');
        if (exists $hChangedValues->{'ClassID'}) {
            UpdateCache($dbi->cacheKey,'ZettleDeleteByClassID', $hInfo->{'ClassID'});
            UpdateCache($dbi->cacheKey,'ZettleDeleteByClassID', $hChangedValues->{'ClassID'});
        }
        $hInfo->{$_} = $hChangedValues->{$_} foreach keys %$hChangedValues;
        my $InfoCache = GetCache($dbi->cacheKey,'InfoZettleDelete');
        $InfoCache->{$GUID} = $hInfo;
    }
    return 1;
}

#========================================================================================
# function     DAL_DeleteZettleDelete
#----------------------------------------------------------------------------------------
# syntax       DAL_DeleteZettleDelete($GUID);
#----------------------------------------------------------------------------------------
# description  removes an entry of table zettledelete
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
#========================================================================================
sub DAL_DeleteZettleDelete ($) {
    my ($GUID) = @_;
    my $cacheKey = GetCurrentDBHandle()->cacheKey;
    my $hInfo = undef;
    if ($DO_CACHE) {
        my $InfoCache = GetCache($cacheKey,'InfoZettleDelete');
        $hInfo = $InfoCache->{$GUID};
    }
    _CallDriver('DeleteZettleDelete', $GUID);
    if ($DO_CACHE) {
        UpdateCache($cacheKey,'InfoZettleDelete', $GUID);
        UpdateCache($cacheKey,'AllZettleDeleteDelete');        
        if (defined $hInfo) {
            UpdateCache($cacheKey,'ZettleDeleteDeleteBySiteID');
            UpdateCache($cacheKey,'ZettleDeleteDeleteByClassID', $hInfo->{'ClassID'});
        } else {
            UpdateCache($cacheKey,'ZettleDeleteDeleteBySiteID');
            UpdateCache($cacheKey,'ZettleDeleteDeleteByClassID');
        }
    }
    return;
}

#========================================================================================
# function     DAL_GetAllPKeyZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $aZettleDeleteIDs = DAL_GetAllPKeyZettleDeletes();
#----------------------------------------------------------------------------------------
# description  returns all primary keys of table zettledelete
#----------------------------------------------------------------------------------------
# return       $aZettleDeleteIDs | GUID of the deleted object - varchar(36) | ref.array.string
#========================================================================================
sub DAL_GetAllPKeyZettleDeletes () {
    return _CallDriver('GetAllPKeyZettleDeletes') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllZettleDelete');
    $CacheRef->{'1'} = _CallDriver('GetAllPKeyZettleDeletes') unless exists $CacheRef->{'1'};
    return [ @{$CacheRef->{'1'}} ];
}

#========================================================================================
# function     DAL_GetAllInfoZettleDeletes
#----------------------------------------------------------------------------------------
# syntax       $ahZettleDeleteIDs = DAL_GetAllInfoZettleDeletes();
#----------------------------------------------------------------------------------------
# description  returns all rows of table zettledelete
#----------------------------------------------------------------------------------------
# return       $ahZettleDeleteIDs | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.array.hash
#========================================================================================
sub DAL_GetAllInfoZettleDeletes ();
sub DAL_GetAllInfoZettleDeletes () {
    return _CallDriver('GetAllInfoZettleDeletes') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'AllZettleDelete');
    if (not exists $CacheRef->{'1'}) {
        my $rResultSet = _CallDriver('GetAllInfoZettleDeletes');
        # make copy to primary key cache
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            push @PrimaryKeySet, $hRow->{'GUID'};
        }
        $CacheRef->{'1'} = \@PrimaryKeySet;
        # make copy to info cache
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoZettleDelete');
        if (_DoFillInfoCache('InfoZettleDelete', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString = $Row->{'GUID'};
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleDelete', @{$CacheRef->{'1'}})) {
        my @ResultSet;
        foreach my $PrimaryKey (@{$CacheRef->{'1'}}) {
            my $Info = DAL_GetInfoZettleDelete($PrimaryKey);
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{'1'};
    return DAL_GetAllInfoZettleDeletes();
}

#========================================================================================
# function     DAL_GetInfoZettleDelete
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoZettleDelete($GUID);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $GUID          | GUID of the deleted object (primary key) | varchar(36)
#----------------------------------------------------------------------------------------
# return       $hResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.hash
#========================================================================================
sub DAL_GetInfoZettleDelete ($) {
    my ($GUID) = @_;
    return _CallDriver('GetInfoZettleDelete', $GUID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef    = GetCache($dbi->cacheKey,'InfoZettleDelete');
    my $CacheString = $GUID;
    $CacheRef->{$CacheString} = _CallDriver('GetInfoZettleDelete', $GUID) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_GetPKeyZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $aGUIDs = DAL_GetPKeyZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  get primary keys of table zettledelete by index i_zettledelete
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $aGUIDs | GUID of the deleted object (primary key - varchar(36)) | ref.array.string
#========================================================================================
sub DAL_GetPKeyZettleDeletesBySiteID ($;$$) {
    my ($SiteID, $ClassID, $DeletedOn) = @_;
    return _CallDriver('GetPKeyZettleDeletesBySiteID', $SiteID, $ClassID, $DeletedOn) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleDeleteBySiteID');
    my $CacheString =  join ('<||>', (map {defined $_ ? $_ : ''} $SiteID, $ClassID, $DeletedOn));
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyZettleDeletesBySiteID', $SiteID, $ClassID, $DeletedOn) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoZettleDeletesBySiteID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_GetInfoZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
#----------------------------------------------------------------------------------------
# description  get full row of table zettledelete by index i_zettledelete
#----------------------------------------------------------------------------------------
# input        $SiteID | Site ID | int
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". (optional) | int
# input        $DeletedOn | Date and time of the deletion on an object (optional) | datetime
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.array.hash
#========================================================================================
sub DAL_GetInfoZettleDeletesBySiteID ($;$$);
sub DAL_GetInfoZettleDeletesBySiteID ($;$$) {
    my ($SiteID, $ClassID, $DeletedOn) = @_;
    return _CallDriver('GetInfoZettleDeletesBySiteID', $SiteID, $ClassID, $DeletedOn) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleDeleteBySiteID');
    my $CacheString =  join ('<||>', (map {defined $_ ? $_ : ''} $SiteID, $ClassID, $DeletedOn));
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoZettleDeletesBySiteID', $SiteID, $ClassID, $DeletedOn);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            push @PrimaryKeySet, $hRow->{'GUID'};
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoZettleDelete');
        if (_DoFillInfoCache('InfoZettleDelete', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString = $Row->{'GUID'};
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleDelete', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoZettleDelete($PrimaryKey);
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoZettleDeletesBySiteID($SiteID, $ClassID, $DeletedOn);
}

#========================================================================================
# function     DAL_GetPKeyZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $raResult = DAL_GetPKeyZettleDeletesByClassID($ClassID);
#----------------------------------------------------------------------------------------
# description  get primary keys of table zettledelete by index i_zettledelete_class
#----------------------------------------------------------------------------------------
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
#----------------------------------------------------------------------------------------
# return       $raGUIDs | GUID of the deleted object (primary key - varchar(36)) | ref.array.string
#========================================================================================
sub DAL_GetPKeyZettleDeletesByClassID ($) {
    my ($ClassID) = @_;
    return _CallDriver('GetPKeyZettleDeletesByClassID', $ClassID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleDeleteByClassID');
    my $CacheString = $ClassID;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyZettleDeletesByClassID', $ClassID) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoZettleDeletesByClassID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_GetInfoZettleDeletesByClassID($ClassID);
#----------------------------------------------------------------------------------------
# description  get full row of table zettledelete by index i_zettledelete_class
#----------------------------------------------------------------------------------------
# input        $ClassID | Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". | int
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GUID          - GUID of the deleted object (primary key)- varchar(36)
#                   <li>SiteID        - Site ID              - int
#                   <li>ClassID       - Class ID. This should be a generic class like "Product" rather than a sub class like "Jacket". - int
#                   <li>PathFromSite  - Object Path relative to the object site. - varchar(1000)
#                   <li>ZettleUUID    - Zettle UUID of the deleted object - varchar(36)
#                   <li>DeletedOn     - Date and time of the deletion on an object - datetime
#               </ul> | ref.array.hash
#========================================================================================
sub DAL_GetInfoZettleDeletesByClassID ($);
sub DAL_GetInfoZettleDeletesByClassID ($) {
    my ($ClassID) = @_;
    return _CallDriver('GetInfoZettleDeletesByClassID', $ClassID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleDeleteByClassID');
    my $CacheString = $ClassID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoZettleDeletesByClassID', $ClassID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            push @PrimaryKeySet, $hRow->{'GUID'};
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey,'InfoZettleDelete');
        if (_DoFillInfoCache('InfoZettleDelete', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString = $Row->{'GUID'};
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleDelete', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoZettleDelete($PrimaryKey);
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoZettleDeletesByClassID($ClassID);
}

#========================================================================================
# function     DAL_InsertZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       DAL_InsertZettleProductExport($hParameter);
#----------------------------------------------------------------------------------------
# description  insert entry in table zettleproductexport
#----------------------------------------------------------------------------------------
# input        $hParameter | with following keys:
#               <ul>
#                   <li>GlobalExportUUID    - Initial export id (part of primary key) - string
#                   <li>BatchNumber         - Number of batch within global export (part of primary key) - int
#                   <li>SiteID              - Site ID - int
#                   <li>ExportType          - Type of export - int
#                   <li>ItemCount           - Number of items included in batch export - int
#                   <li>BatchExportStatus   - Batch export status - int
#                   <li>StartedOn           - Date and time when batch export started (optional. default: current time) - datetime
#               </ul> | ref.hash
#========================================================================================
sub DAL_InsertZettleProductExport ($) {
    my ($hParameter) = @_;
    $hParameter->{'StartedOn'} = GetCurrentDateTime() unless defined $hParameter->{'StartedOn'};
    _CallDriver('InsertZettleProductExport', $hParameter);
    if ($DO_CACHE) {
        my $dbi = GetCurrentDBHandle();
        foreach my $key (keys %$hParameter) {
            $hParameter->{$key} = $dbi->encodeSpaces($hParameter->{$key});
        }
        UpdateCache($dbi->cacheKey,'InfoZettleProductExport', join('<||>', @$hParameter{qw (GlobalExportUUID BatchNumber)}), $hParameter);
        UpdateCache($dbi->cacheKey,'ZettleProductExportByGlobalExportUUID', $hParameter->{'GlobalExportUUID'});
        UpdateCache($dbi->cacheKey,'ZettleProductExportByStatus', join('<||>', @$hParameter{qw (GlobalExportUUID BatchExportStatus)}));
    }
    return;
}

#========================================================================================
# function     DAL_UpdateZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       DAL_UpdateZettleProductExport($GlobalExportUUID, $BatchNumber, $hParameter);
#----------------------------------------------------------------------------------------
# description  updates an entry of table zettleproductexport
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchNumber | Number of batch within global export (part of primary key) | int
# input        $hParameter | with following keys:
#               <ul>
#                   <li>BatchExportUUID   - Batch export UUID - varchar(36)
#                   <li>BatchExportStatus - New Value for batch export status - int
#                   <li>UpdatedOn         - Datetime when export was last updated (optional. default: current datetime) - datetime
#                   <li>ItemCount         - Number of items included in batch export - int
#                   <li>ErrorType         - Error identifier - varchar(80)
#                   <li>ErrorMessage      - Detailed error message - varchar(255)
#               </ul>| ref.hash
#========================================================================================
sub DAL_UpdateZettleProductExport ($$$) {
    my ($GlobalExportUUID, $BatchNumber, $hParameter) = @_;
    my $hInfo = DAL_GetInfoZettleProductExport($GlobalExportUUID, $BatchNumber);
    my $hChangedValues = {};
    my $dbi = GetCurrentDBHandle();

    $hParameter->{'UpdatedOn'} = GetCurrentDateTime() unless defined $hParameter->{'UpdatedOn'};
    foreach my $ChangedKey (qw ( BatchExportUUID BatchExportStatus UpdatedOn ItemCount ErrorType ErrorMessage )) {
        next unless exists $hParameter->{$ChangedKey};
        next if not defined $hInfo->{$ChangedKey} and not defined $hParameter->{$ChangedKey};
        if ( defined $hParameter->{$ChangedKey}
             and not defined $hInfo->{$ChangedKey}
             or
             not defined $hParameter->{$ChangedKey}
             and defined $hInfo->{$ChangedKey}
        ) {
            $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey});
            next;
        }
        next if $hInfo->{$ChangedKey} eq $hParameter->{$ChangedKey};
        $hChangedValues->{$ChangedKey} = $dbi->encodeSpaces($hParameter->{$ChangedKey})
    }
    return 0 unless scalar keys %$hChangedValues;
    
    _CallDriver('UpdateZettleProductExport', $GlobalExportUUID, $BatchNumber, $hChangedValues);

    if($DO_CACHE) {
        # Copy old values and updates ones to a new hash which is used to update the cache.
        my $hMergedCacheInfo = { %$hInfo, %$hChangedValues };
        UpdateCache($dbi->cacheKey,'InfoZettleProductExport', join('<||>', $GlobalExportUUID, $BatchNumber), $hMergedCacheInfo);
        UpdateCache($dbi->cacheKey,'ZettleProductExportByGlobalExportUUID', $GlobalExportUUID);
        if (exists $hChangedValues->{'BatchExportStatus'}) {
            UpdateCache($dbi->cacheKey,'ZettleProductExportByStatus', join('<||>', $GlobalExportUUID, $hInfo->{'BatchExportStatus'}));
            UpdateCache($dbi->cacheKey,'ZettleProductExportByStatus', join('<||>', $GlobalExportUUID, $hChangedValues->{'BatchExportStatus'}));
        }
    }
    return 1;
}

#========================================================================================
# function     DAL_GetInfoZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoZettleProductExport($GlobalExportUUID, $BatchNumber);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key) | varchar(36)
# input        $BatchNumber | Number of batch within global export (part of primary key) | int
#----------------------------------------------------------------------------------------
# return       $hResult | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul>| ref.hash
#========================================================================================
sub DAL_GetInfoZettleProductExport ($$) {
    my ($GlobalExportUUID, $BatchNumber) = @_;
    return _CallDriver('GetInfoZettleProductExport', $GlobalExportUUID, $BatchNumber) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'InfoZettleProductExport');
    my $CacheString = join('<||>', $GlobalExportUUID, $BatchNumber);
    $CacheRef->{$CacheString} = _CallDriver('GetInfoZettleProductExport', $GlobalExportUUID, $BatchNumber) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_DeleteZettleProductExport
#----------------------------------------------------------------------------------------
# syntax       DAL_DeleteZettleProductExport($Days);
#----------------------------------------------------------------------------------------
# description  Delete zettleproductexport entries older than $Days.
#----------------------------------------------------------------------------------------
# input        $Days | number of days | integer
#========================================================================================
sub DAL_DeleteZettleProductExport ($) {
    my ($Days) = @_;
    _CallDriver('DeleteZettleProductExport', $Days);
    if ($DO_CACHE) {
        my $cacheKey = GetCurrentDBHandle()->cacheKey;
        UpdateCache($cacheKey,'InfoZettleProductExport');
        UpdateCache($cacheKey,'ZettleProductExportByGlobalExportUUID');
        UpdateCache($cacheKey,'ZettleProductExportByStatus');
    }
    return;
}

#========================================================================================
# function     DAL_GetInfoZettleProductExportByGlobalExportUUID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_GetInfoZettleProductExportByGlobalExportUUID($GlobalExportUUID);
#----------------------------------------------------------------------------------------
# description  Retrieve all entries with the same GlobalExportUUID.
#               Table zettleproductexport by index i_zettleproductexport_globalexportuuid_batchexportstatus.
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global Export UUID (part of primary key) | varchar(36)
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul>| ref.array.hash
#========================================================================================
sub DAL_GetInfoZettleProductExportByGlobalExportUUID ($);
sub DAL_GetInfoZettleProductExportByGlobalExportUUID ($) {
    my ($GlobalExportUUID) = @_;
    return _CallDriver('GetInfoZettleProductExportByGlobalExportUUID', $GlobalExportUUID) unless $DO_CACHE;
    
    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleProductExportByGlobalExportUUID');
    my $CacheString = $GlobalExportUUID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoZettleProductExportByGlobalExportUUID', $GlobalExportUUID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (GlobalExportUUID BatchNumber)} = @$hRow{qw (GlobalExportUUID BatchNumber)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey, 'InfoZettleProductExport');
        if (_DoFillInfoCache('InfoZettleProductExport', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join ('<||>', @$Row{qw (GlobalExportUUID BatchNumber)});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleProductExport', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoZettleProductExport($PrimaryKey->{'GlobalExportUUID'}, $PrimaryKey->{'BatchNumber'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoZettleProductExportByGlobalExportUUID($GlobalExportUUID);
}

#========================================================================================
# function     DAL_GetInfoZettleProductExportBySiteID
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_GetInfoZettleProductExportBySiteID($SiteID);
#----------------------------------------------------------------------------------------
# description  Retrieve all entries by SiteID.
#               Table zettleproductexport by index i_zettleproductexport_siteid.
#----------------------------------------------------------------------------------------
# input        $SiteID | ID of a shop | int
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul>| ref.array.hash
#========================================================================================
sub DAL_GetInfoZettleProductExportBySiteID ($);
sub DAL_GetInfoZettleProductExportBySiteID ($) {
    my ($SiteID) = @_;
    return _CallDriver('GetInfoZettleProductExportBySiteID', $SiteID) unless $DO_CACHE;
    
    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleProductExportBySiteID');
    my $CacheString = $SiteID;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoZettleProductExportBySiteID', $SiteID);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (GlobalExportUUID BatchNumber)} = @$hRow{qw (GlobalExportUUID BatchNumber)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey, 'InfoZettleProductExport');
        if (_DoFillInfoCache('InfoZettleProductExport', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString = join ('<||>', @$Row{qw (GlobalExportUUID BatchNumber)});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleProductExport', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoZettleProductExport($PrimaryKey->{'GlobalExportUUID'}, $PrimaryKey->{'BatchNumber'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoZettleProductExportBySiteID($SiteID);
}

#========================================================================================
# function     DAL_GetInfoZettleProductExportByStatus
#----------------------------------------------------------------------------------------
# syntax       $ahResult = DAL_GetInfoZettleProductExportByStatus($GlobalExportUUID, $BatchExportStatus);
#----------------------------------------------------------------------------------------
# description  Retrieve all entries with the same GlobalExportUUID and BatchExportStatus.
#               Table zettleproductexport by index i_zettleproductexport_globalexportuuid_batchexportstatus.
#----------------------------------------------------------------------------------------
# input        $GlobalExportUUID | Global export UUID (part of primary key)  | varchar(36)
# input        $BatchExportStatus | Batch export status (see EXPORT_STATUS values
#                                    in <package DE_EPAGES::Zettle::API::Constants>) | int
#----------------------------------------------------------------------------------------
# return       $ahResult | with following keys:
#               <ul>
#                   <li>GlobalExportUUID   - Global export UUID (part of primary key) - string
#                   <li>BatchNumber        - Number of batch within global export (part of primary key) - int
#                   <li>SiteID             - Site ID - int
#                   <li>ExportType         - Type of export - int
#                   <li>ItemCount          - Number of items included in batch export - int
#                   <li>BatchExportStatus  - Status of batch export - int
#                   <li>BatchExportUUID    - Batch export UUID (part of primary key) - string
#                   <li>StartedOn          - Date and time when batch export started (optional. default: current time) - datetime
#                   <li>UpdatedOn          - Date when entry was last updated (optional) - datetime
#                   <li>ErrorType          - Error identifier - varchar(80)
#                   <li>ErrorMessage       - Detailed error message - varchar(255)
#               </ul>| ref.array.hash
#========================================================================================
sub DAL_GetInfoZettleProductExportByStatus ($$);
sub DAL_GetInfoZettleProductExportByStatus ($$) {
    my ($GlobalExportUUID, $BatchExportStatus) = @_;
    return _CallDriver('GetInfoZettleProductExportByStatus', $GlobalExportUUID, $BatchExportStatus) unless $DO_CACHE;
    
    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey,'ZettleProductExportByStatus');
    my $CacheString = join('<||>', $GlobalExportUUID, $BatchExportStatus);
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoZettleProductExportByStatus', $GlobalExportUUID, $BatchExportStatus);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (GlobalExportUUID BatchNumber)} = @$hRow{qw (GlobalExportUUID BatchNumber)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey, 'InfoZettleProductExport');
        if (_DoFillInfoCache('InfoZettleProductExport', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString = join ('<||>', @$Row{qw (GlobalExportUUID BatchNumber)});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoZettleProductExport', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoZettleProductExport($PrimaryKey->{'GlobalExportUUID'}, $PrimaryKey->{'BatchNumber'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoZettleProductExportByStatus($GlobalExportUUID, $BatchExportStatus);
}

1;
